#include "AntiStunLockCompositeActor.h"

AntiStunLockCompositeActor::AntiStunLockCompositeActor(
	CoreUtils* coreUtils,
	Console* console,
	FallingDownActor* fallingDownActor,
	SingleActionActor* hitRecoveringActor,
	MovingActor* movingActor,
	SingleActionActor* singleHitActor,
	ComboActor* comboActor,
	SingleActionActor* healActor,
	SingleActionActor* mysticalActor,
	ChargingActor* chargingActor
	) : RandomlyFallingCompositeActor (
		coreUtils,
		console,
		fallingDownActor,
		hitRecoveringActor,
		movingActor,
		singleHitActor,
		comboActor,
		healActor,
		mysticalActor,
		chargingActor
		) {
		
	m_coreUtils = coreUtils;
	this->ResetArray();
	console->Debug( "AntiStunLockCompositeActor: at %p", this );
}

AntiStunLockCompositeActor::~AntiStunLockCompositeActor() {
	m_console->Debug( "~AntiStunLockCompositeActor: at %p", this );
}

void AntiStunLockCompositeActor::ResetArray() {
	for( u8 i = 0; i < HIT_THIS_MANY_TIMES; i++ ) {
		m_hitArray[ i ] = Coords_F( -1000.0f, -1000.f );
	}
}

void AntiStunLockCompositeActor::ShiftLeftAndStorePosition() {
	for( u8 i = 0; i < HIT_THIS_MANY_TIMES - 1; i++ ) {
		m_hitArray[ i ] = m_hitArray[ i + 1 ];
	}
	m_hitArray[ HIT_THIS_MANY_TIMES - 1 ] = m_movingActor->GetPosition();
}

void AntiStunLockCompositeActor::RecoverFromHit() {

	this->ShiftLeftAndStorePosition();
	
	// have we not been hit enough times?
	if( m_hitArray[ 0 ].x == -100.0f ) {
		// no - perform default behaviour
		RandomlyFallingCompositeActor::RecoverFromHit();
		return;
	}

	bool allEqual = true;
	for( u8 i = 0; i < HIT_THIS_MANY_TIMES - 1; i++ ) {
		if( m_hitArray[ i ] != m_hitArray[ i + 1 ] ) {
			allEqual = false;
			break;
		}
	}
	
	// have all hits been registered at the same location?
	if( allEqual && this->CanFallDownOrRecoverFromHit() ) {
		this->FallDown();
		return;
	}

	// default behaviour
	RandomlyFallingCompositeActor::RecoverFromHit();
}

void AntiStunLockCompositeActor::FallDown() {
	this->ResetArray();
	RandomlyFallingCompositeActor::FallDown();
}
