#ifndef ANTISTUNLOCK_COMPOSITEACTOR_H
#define ANTISTUNLOCK_COMPOSITEACTOR_H

#include "FallingDownActor.h"
#include "MovingActor.h"
#include "SingleActionActor.h"
#include "ComboActor.h"
#include "ChargingActor.h"
#include "RandomlyFallingCompositeActor.h"

/*
	A composite actor that behaves differently after being hit a set
	number of times without having moved (changed position).
	
	The purpose of this is to prevent unfinished combos from 
	stun locking bosses.
*/
class AntiStunLockCompositeActor : public RandomlyFallingCompositeActor {

	private:
	
	static const u8 HIT_THIS_MANY_TIMES = 4;

	Coords_F m_hitArray[ HIT_THIS_MANY_TIMES ]; // positions of last few hits
	CoreUtils* m_coreUtils;
	
	void ResetArray();
	void ShiftLeftAndStorePosition();
	
	public:
	
	AntiStunLockCompositeActor(
		CoreUtils* coreUtils,
		Console* console,
		FallingDownActor* fallingDownActor,
		SingleActionActor* hitRecoveringActor,
		MovingActor* movingActor,
		SingleActionActor* singleHitActor,
		ComboActor* comboActor,
		SingleActionActor* healActor,
		SingleActionActor* mysticalActor,
		ChargingActor* chargingActor
		);
		
	virtual ~AntiStunLockCompositeActor();
	
	void RecoverFromHit();
	void FallDown();
};

#endif