#include "ChargingActor.h"

ChargingActor::ChargingActor( 
	Console* console,
	Animator* animator
	) {
	
	m_console = console;
	m_animator = animator;
	
	this->Cancel();
}

ChargingActor::~ChargingActor() {
}

void ChargingActor::Charge( 
	Coords_F destination,
	float lengthUnitsPerFrame
	) {
	
	if( m_isCharging ) {
		return;
	}
	
	m_lengthUnitsPerFrame = lengthUnitsPerFrame;
	
	m_animator->ChangePhase( ENEMY_PHASE_CHARGE );
	
	Coords_F position = m_animator->GetSprite()->GetPosition();
	m_delta = destination - position;
	
	// total travel distance
	float distanceToTarget = m_delta.GetLength();
	// overshoot by a bit
	distanceToTarget = distanceToTarget + 64.0f;
	
	// convert to a vector that's as long as this actor should travel in a frame,
	// oriented towards its target
	m_delta.NormalizeAndMultiplyBy( m_lengthUnitsPerFrame );
	
	// calculate how many frames it will take to get there
	m_totalFrameCount = (u8)( distanceToTarget / m_lengthUnitsPerFrame );
	m_console->Debug( "Charging!" );
	
	// turn sprite if necessary
	OAMSprite* sprite = m_animator->GetSprite();
	if( m_delta.x < 0 ) {
		sprite->SetIsFacingRight( false );
	} else if( m_delta.x > 0 ) {
		sprite->SetIsFacingRight( true );
	}
	
	m_isCharging = true;
}

void ChargingActor::Render() {
	
	if( !m_isCharging ) {
		return;
	}
	
	m_animator->AdvanceFrame();
	
	Coords_F spritePosition = m_animator->GetSprite()->GetPosition();
	Coords_F newPosition = spritePosition + m_delta;
	m_animator->GetSprite()->MoveTo( newPosition );
	
	m_animator->Render();
	
	m_frameCount++;
	
	// have we finished?
	if( m_frameCount >= m_totalFrameCount ) {
		this->Cancel();
	}
}

bool ChargingActor::IsInterruptibleBySelf() {
	return !m_isCharging;
}

bool ChargingActor::IsInterruptibleByOther() {
	return true;
}

void ChargingActor::Cancel() {	
	m_frameCount = 0;
	m_totalFrameCount = 0;
	m_isCharging = false;
}