#ifndef __CHARGING_ACTOR_H
#define __CHARGING_ACTOR_H

#include "../core/core.h"
#include "../animators/animators.h"

/*
	An entity that can perform the following actions:
		- charge towards a target location
	
	IMPORTANT: This Actor does NOT delete its animator
*/
class ChargingActor {

	private:
	
	Console* m_console;
	Animator* m_animator;
	float m_lengthUnitsPerFrame; // charging speed in units per frame
	
	Coords_F m_delta;
	
	u8 m_totalFrameCount; // for how many frames are we animating
	u8 m_frameCount; // since the action was initiated
	bool m_isCharging;
	
	public:
	
	ChargingActor( 
		Console* console,
		Animator* animator
		);
	
	virtual ~ChargingActor();
	
	void Charge( 
		Coords_F destination,
		float lengthUnitsPerFrame
	);

	bool IsInterruptibleBySelf();
	bool IsInterruptibleByOther();
	
	void Cancel();
	
	void Render();

};

#endif