#include "ComboActor.h"

ComboActor::ComboActor(
	Console* console,
	SingleActionActor** actors,
	u8 actorCount
	) {

	m_console = console;
	m_actors = actors;
	m_actorCount = actorCount;
	
	// inactive state
	m_currentActor = m_actorCount;
}
	
ComboActor::~ComboActor() {
	for( u8 i = 0; i < m_actorCount; i++ ) {
		delete m_actors[ i ];
	}
	
	delete m_actors;
}
	
bool ComboActor::IsInterruptibleBySelf() {
	if( m_currentActor >= m_actorCount ) {
		return true;
	}
	
	return m_actors[ m_currentActor ]->IsInterruptibleBySelf();
}

bool ComboActor::IsInterruptibleByOther() {
	if( m_currentActor >= m_actorCount ) {
		return true;
	}
	
	return m_actors[ m_currentActor ]->IsInterruptibleByOther();
}
	
void ComboActor::PerformCombo( 
	Event* continuationEvent,
	Event* animationProgressReachedEvent,
	float animationProgress
	) {
	
	m_animationProgressReachedEvent = animationProgressReachedEvent;
	m_animationProgress = animationProgress;
	m_continuationEvent = continuationEvent;
	
	// start up first actor
	m_currentActor = 0;
	m_actors[ m_currentActor ]->PerformAction(
		m_animationProgressReachedEvent,
		m_animationProgress
		);
}
	
void ComboActor::Render() {
	
	if( m_currentActor >= m_actorCount ) {
		return;
	}

	bool isInterruptible = 
		m_actors[ m_currentActor ]->IsInterruptibleBySelf() && 
		m_actors[ m_currentActor ]->IsInterruptibleByOther();
		
	if( !isInterruptible ) {
	
		m_actors[ m_currentActor ]->Render();
		
		isInterruptible = 
			m_actors[ m_currentActor ]->IsInterruptibleBySelf() && 
			m_actors[ m_currentActor ]->IsInterruptibleByOther();

		// is this actor done?
		if( isInterruptible ) {
			
			m_currentActor++;
			
			// are we done overall?
			if( m_currentActor >= m_actorCount ) {
				return;
			}
			
			// should we continue our combo?
			if( !m_continuationEvent->IsSet() ) {
				return;
			}
			
			// start up the next actor
			m_actors[ m_currentActor ]->PerformAction(
				m_animationProgressReachedEvent,
				m_animationProgress
				);
		}
	}
}

void ComboActor::Cancel() {
	
	if( m_currentActor < m_actorCount ) {
		m_actors[ m_currentActor ]->Cancel();
	}

	m_currentActor = m_actorCount;
}