#ifndef COMPOSITEACTOR_H
#define COMPOSITEACTOR_H

#include "FallingDownActor.h"
#include "MovingActor.h"
#include "SingleActionActor.h"
#include "ComboActor.h"
#include "ChargingActor.h"

/*
	Aggregates actors
*/
class CompositeActor {

	private:
	
	FallingDownActor* m_fallingDownActor;
	SingleActionActor* m_hitRecoveringActor;
	SingleActionActor* m_singleHitActor;
	ComboActor* m_comboActor;
	SingleActionActor* m_healActor;
	SingleActionActor* m_mysticalActor;
	ChargingActor* m_chargingActor;
	
	bool m_hasDied;
	bool m_canBeRemoved;
	bool m_displacementTendencyIsToTheRight;
	
	bool IsInterruptibleBySelf();
	bool IsInterruptibleByOther();
	
	bool TryRenderChargingActor();
	bool TryRenderFallingDownActor();
	bool TryRenderHitRecoveringActor();
	bool TryRenderSingleHitActor();
	bool TryRenderComboActor();
	bool TryRenderHealActor();
	bool TryRenderMysticalActor();
	// renders an actor and if the actor finishes its action, switches to a standing state
	bool Helper_TryRenderSingle( SingleActionActor* actor );
	
	void CancelAllActors();
	
	protected:
	
	Console* m_console;
	MovingActor* m_movingActor;
	
	public:
	
	CompositeActor(
		Console* console,
		FallingDownActor* fallingDownActor,
		SingleActionActor* hitRecoveringActor,
		MovingActor* movingActor,
		SingleActionActor* singleHitActor,
		ComboActor* comboActor,
		SingleActionActor* healActor,
		SingleActionActor* mysticalActor,
		ChargingActor* chargingActor
		);
		
	virtual ~CompositeActor();
	
	void RelocateTo( Coords_F position );
	void SetIsFacingRight( bool isFacingRight );
	
	// moving and standing
	bool CanMoveOrStand();
	void MoveBy( Coords_F deltaPosition );
	void Stand();
	
	// being hit and falling down and then getting up, or just recovering from a hit
	bool CanFallDownOrRecoverFromHit();
	virtual void FallDown();
	virtual void RecoverFromHit();
	
	// striking once
	bool CanPerformSingleHit();
	void PerformSingleHit(
		Event* animationProgressReachedEvent,
		float animationProgress
		);
	
	// performing a combo
	bool CanPerformCombo();
	void PerformCombo(
		Event* continuationEvent,
		Event* animationProgressReachedEvent,
		float animationProgress
		);
	
	// player special moves
	bool CanHealOrMystical();
	void Heal();
	void Mystical();
	bool IsPerformingMystical();
	
	// enemy charging
	bool CanCharge();
	void Charge( 
		Coords_F destination,
		float lengthUnitsPerFrame
		);
	bool IsCharging();

	// in case the actor has to be displaced, it will be displaced in this direction
	void SetDisplacementTendency( bool isTowardsRight );
	void SetHasDied();
	bool CanBeRemoved();
	virtual void Render();
};

#endif