#include "CompositeActorFactory.h"
	
CompositeActorFactory::CompositeActorFactory( 
	CoreUtils* coreUtils,
	Console* console,
	AnimatorFactory* animatorFactory,
	FallingDownActorFactory* fallingDownActorFactory,
	HitRecoveringActorFactory* hitRecoveringActorFactory,
	SingleActionActorFactory* singleActionActorFactory,
	ComboActorFactory* comboActorFactory,
	ChargingActorFactory* chargingActorFactory
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	m_animatorFactory = animatorFactory;
	m_fallingDownActorFactory = fallingDownActorFactory;
	m_hitRecoveringActorFactory = hitRecoveringActorFactory;
	m_singleActionActorFactory = singleActionActorFactory;
	m_comboActorFactory = comboActorFactory;
	m_chargingActorFactory = chargingActorFactory;
}

CompositeActorFactory::~CompositeActorFactory() {
}

UtilitiesCompositeActor* CompositeActorFactory::MakePlayer() {
	Animator* animator = m_animatorFactory->Create( ANIMATOR_PLAYER );
			
	FallingDownActor* fallingDownActor = m_fallingDownActorFactory->Create( animator, true );
	SingleActionActor* hitRecoveringActor = m_hitRecoveringActorFactory->Create( animator );
	MovingActor* movingActor = new MovingActor( m_console, animator, true );
	SingleActionActor* singleHitActor = 
		m_singleActionActorFactory->Create( animator, PHASE_SINGLE_HIT, PHASE_SINGLE_HIT );
	ComboActor* comboActor = m_comboActorFactory->Create( animator );
	SingleActionActor* healActor = 
		m_singleActionActorFactory->CreateUninterruptible( animator, PLAYER_PHASE_HEAL );
	SingleActionActor* mysticalActor = 
		m_singleActionActorFactory->CreateUninterruptible( animator, PLAYER_PHASE_MYSTICAL );
	ChargingActor* chargingActor =  m_chargingActorFactory->Create( animator );
	
	UtilitiesCompositeActor* actor = new HitCountingCompositeActor(
		m_coreUtils,
		m_console,
		fallingDownActor,
		hitRecoveringActor,
		movingActor,
		singleHitActor,
		comboActor,
		healActor,
		mysticalActor,
		chargingActor
		);
	
	return actor;
}

UtilitiesCompositeActor* CompositeActorFactory::MakeEnemy( 
	AnimatorTypes animatorType,
	bool useAntiStunLock
	) {
	Animator* animator = m_animatorFactory->Create( animatorType );
			
	FallingDownActor* fallingDownActor = m_fallingDownActorFactory->Create( animator, false );
	SingleActionActor* hitRecoveringActor = m_hitRecoveringActorFactory->Create( animator );
	MovingActor* movingActor = new MovingActor( m_console, animator, true );
	SingleActionActor* singleHitActor = 
		m_singleActionActorFactory->Create( animator, PHASE_SINGLE_HIT, ENEMY_PHASE_ALTERNATE_HIT );
	ComboActor* comboActor = m_comboActorFactory->Create( animator );
	SingleActionActor* healActor = 
		m_singleActionActorFactory->CreateUninterruptible( animator, PLAYER_PHASE_HEAL );
	SingleActionActor* mysticalActor = 
		m_singleActionActorFactory->CreateUninterruptible( animator, PLAYER_PHASE_MYSTICAL );
	ChargingActor* chargingActor =  m_chargingActorFactory->Create( animator );
	
	UtilitiesCompositeActor* actor;
		
	if( useAntiStunLock ) {
		actor = new AntiStunLockCompositeActor(
			m_coreUtils,
			m_console,
			fallingDownActor,
			hitRecoveringActor,
			movingActor,
			singleHitActor,
			comboActor,
			healActor,
			mysticalActor,
			chargingActor
			);
	} else {
		actor = new RandomlyFallingCompositeActor(
			m_coreUtils,
			m_console,
			fallingDownActor,
			hitRecoveringActor,
			movingActor,
			singleHitActor,
			comboActor,
			healActor,
			mysticalActor,
			chargingActor
			);
	}
	
	return actor;
}

UtilitiesCompositeActor* CompositeActorFactory::Create( ActorTypes type ) {

	switch( type ) {	
		case ACTOR_PLAYER:
			return this->MakePlayer();
			
		case ACTOR_SMECHER:
			return this->MakeEnemy( ANIMATOR_SMECHER, false );
		case ACTOR_SMECHER_BLUE:
			return this->MakeEnemy( ANIMATOR_SMECHER_BLUE, false );
		case ACTOR_SMECHER_GREEN:
			return this->MakeEnemy( ANIMATOR_SMECHER_GREEN, false );
		case ACTOR_SMECHER_PURPLE:
			return this->MakeEnemy( ANIMATOR_SMECHER_PURPLE, false );
		case ACTOR_SMECHER_RED_NOSTUNLOCK:
			return this->MakeEnemy( ANIMATOR_SMECHER_RED, true );
			
		case ACTOR_CUTITAR:
			return this->MakeEnemy( ANIMATOR_CUTITAR, false );
		case ACTOR_CUTITAR_NOSTUNLOCK:
			return this->MakeEnemy( ANIMATOR_CUTITAR, true );
			
		case ACTOR_MASCAT:
			return this->MakeEnemy( ANIMATOR_MASCAT, false );
		case ACTOR_MASCAT_GREEN:
			return this->MakeEnemy( ANIMATOR_MASCAT_GREEN, false );
		case ACTOR_MASCAT_RED_NOSTUNLOCK:
			return this->MakeEnemy( ANIMATOR_MASCAT_RED, true );
			
		case ACTOR_MEISS:
			return this->MakeEnemy( ANIMATOR_MEISS, false );
		case ACTOR_MEISS_DARK:
			return this->MakeEnemy( ANIMATOR_MEISS_DARK, false );
		case ACTOR_MEISS_NOSTUNLOCK:
			return this->MakeEnemy( ANIMATOR_MEISS, true );
	}
	
	m_console->Error( "[CompositeActorFactory]Create: Invalid Actor type!" );
	return NULL;
}
