#include "FallingDownActor.h"

FallingDownActor::FallingDownActor( 
	CoreUtils* coreUtils,
	Console* console,
	Animator* animator,
	Coords_F* parabola,
	u8 parabolaFrameCount,
	u8 onGroundFrameCount,
	bool keepOnScreen
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	m_animator = animator;
	m_parabola = parabola;
	m_parabolaFrameCount = parabolaFrameCount;
	m_onGroundFrameCount = onGroundFrameCount;
	m_keepOnScreen = keepOnScreen;
	
	m_frameCount = 0;
	m_isFallingDown = false;
	m_isOnTheGround = false;
}

FallingDownActor::~FallingDownActor() {
}

void FallingDownActor::FallDown( bool isTowardsRight ) {
	
	if( m_isFallingDown ) {
		return;
	}
	
	if( m_isOnTheGround ) {
		return;
	}
	
	m_isTowardsRight = isTowardsRight;
	m_frameCount = 0;
	m_animator->ChangePhase( PHASE_FALLING_DOWN );
	
	m_initialSpriteCoords = m_animator->GetSprite()->GetPosition();
	
	float tenths = (float)( m_coreUtils->Random( 6 ) );
	m_randomMultiplierX = 1.0f + ( tenths / 10.0f );
	
	m_isFallingDown = true;
	m_isOnTheGround = false;
}

void FallingDownActor::Render() {
	
	if( m_isFallingDown ) {
		this->RenderFallingDownState();
		return;
	}
	
	if( m_isOnTheGround ) {
		this->RenderOnGroundState();
		return;
	}
}

bool FallingDownActor::IsInterruptible() {
	return !m_isFallingDown && !m_isOnTheGround;
}

void FallingDownActor::RenderFallingDownState() {
	
	m_animator->AdvanceFrame();

	float parabolaX = m_parabola[ m_frameCount ].x;
	if( !m_isTowardsRight ) {
		// fall towards the left
		parabolaX = parabolaX * -1.0f;
	}
	
	// apply the random X multiplier to create diversity in groups of falling enemies
	parabolaX = parabolaX * m_randomMultiplierX;
	
	float newX = m_initialSpriteCoords.x + parabolaX;
	
	float newY = m_initialSpriteCoords.y + 
		m_parabola[ m_frameCount ].y -
		m_parabola[ 0 ].y;
	
	Coords_F newPosition( newX, newY );
	// prevent off-screen if we need to
	if( m_keepOnScreen ) {
		newPosition.x = fmaxf( newPosition.x, LEFT_BOUND );
		newPosition.x = fminf( newPosition.x, RIGHT_BOUND );
	}
	
	// we use an unchecked move because the sprite can move above the top boundary
	// when it is in the air.
	// we can bypass the boundary checks in this manner because we guarantee that
	// the final Y position will be equal to the initial Y position, and thus, 
	// within bounds
	m_animator->GetSprite()->MoveToUnchecked( newPosition );
	
	m_animator->Render();
	
	m_frameCount++;
	
	// have we finished falling down?
	if( m_frameCount >= m_parabolaFrameCount ) {
		// ensure the sprite's Y position hasn't changed from when we started animating
		Coords_F finalPosition(
			m_animator->GetSprite()->GetPosition().x,
			m_initialSpriteCoords.y
			);
			
		m_animator->GetSprite()->MoveTo( finalPosition );
		
		// initiate "on ground" state
		m_frameCount = 0;
		m_isFallingDown = false;
		m_isOnTheGround = true;
	}
}

void FallingDownActor::RenderOnGroundState() {
	
	m_animator->Render();
	
	m_frameCount++;
	
	// have we finished being on the ground?
	if( m_frameCount >= m_onGroundFrameCount ) {
		this->Cancel();
	}
}

void FallingDownActor::Cancel() {
	
	m_frameCount = 0;
	m_isFallingDown = false;
	m_isOnTheGround = false;
}