#ifndef __FALLINGDOWNACTOR_H
#define __FALLINGDOWNACTOR_H

#include "../core/core.h"
#include "../animators/animators.h"

/*
	An entity that can perform the following actions:
		- fly through the air after being hit
		- lay on the ground
	
	IMPORTANT: This Actor does NOT delete its animator
*/
class FallingDownActor {

	private:
	
	static const float LEFT_BOUND = -20.0f;
	static const float RIGHT_BOUND = 211.0f;
	
	CoreUtils* m_coreUtils;
	Console* m_console;
	Animator* m_animator;
	
	// deleted by factory
	Coords_F* m_parabola;
	
	u8 m_parabolaFrameCount; // how many frames it takes to animate falling down
	u8 m_onGroundFrameCount; // how many frames it takes before it can get up
	bool m_isTowardsRight; // whether we're falling towards the right
	bool m_keepOnScreen; // whether animator should be constrained to remain on screen
	float m_randomMultiplierX; // applies on every delta X to create diversity in groups
							   // of enemies falling down together
							   
	u8 m_frameCount; // since the fall was initiated
	Coords_F m_initialSpriteCoords;
	
	bool m_isFallingDown;
	bool m_isOnTheGround;
	
	void RenderFallingDownState();
	void RenderOnGroundState();
	
	public:
	
	FallingDownActor( 
		CoreUtils* coreUtils,
		Console* console,
		Animator* animator,
		Coords_F* parabola,
		u8 parabolaFrameCount,
		u8 onGroundFrameCount,
		bool keepOnScreen
		);
	
	virtual ~FallingDownActor();
	
	void FallDown( bool isTowardsRight );
	
	// true when this actor can be interrupted
	bool IsInterruptible();
	void Cancel();
	
	void Render();

};

#endif