#include "FallingDownActorFactory.h"

	
FallingDownActorFactory::FallingDownActorFactory( 
	CoreUtils* coreUtils,
	Console* console
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	
	// calculate parabola values
	float halfDistance = TOTAL_FALL_HORIZONTAL_DISTANCE / 2.0f;
	
	u8 frameCountWithoutBounce = PARABOLA_FRAME_COUNT - BOUNCE_FRAME_COUNT;
	
	float xIncrement = TOTAL_FALL_HORIZONTAL_DISTANCE / (float) ( frameCountWithoutBounce - 1 );
	
	// fill in values to describe a parabolic trajectory
	for( u8 i = 0; i < frameCountWithoutBounce; i++ ) {

		float x = xIncrement * (float)i;
		float y = 0.037f * ( x - halfDistance ) * ( x - halfDistance );
		
		m_parabola[ i ].x = x;
		m_parabola[ i ].y = y;
	}
	
	// append a few entries for "bouncing" after it hit the ground
	for( u8 i = frameCountWithoutBounce; i < PARABOLA_FRAME_COUNT; i++ ) {
		
		float x = xIncrement * (float)i;
		float y = m_parabola[ i - 2 ].y;
		
		m_parabola[ i ].x = x;
		m_parabola[ i ].y = y;
	}
}

FallingDownActorFactory::~FallingDownActorFactory() {
}
	
FallingDownActor* FallingDownActorFactory::Create( 
	Animator* animator,
	bool keepOnScreen
	) {

	return new FallingDownActor( 
		m_coreUtils,
		m_console, 
		animator,
		&m_parabola[ 0 ],
		PARABOLA_FRAME_COUNT,
		ON_GROUND_FRAME_COUNT,
		keepOnScreen
		);
}
