#include "HitCountingCompositeActor.h"

HitCountingCompositeActor::HitCountingCompositeActor(
	CoreUtils* coreUtils,
	Console* console,
	FallingDownActor* fallingDownActor,
	SingleActionActor* hitRecoveringActor,
	MovingActor* movingActor,
	SingleActionActor* singleHitActor,
	ComboActor* comboActor,
	SingleActionActor* healActor,
	SingleActionActor* mysticalActor,
	ChargingActor* chargingActor
	) : RandomlyFallingCompositeActor (
		coreUtils,
		console,
		fallingDownActor,
		hitRecoveringActor,
		movingActor,
		singleHitActor,
		comboActor,
		healActor,
		mysticalActor,
		chargingActor
		) {
		
	m_coreUtils = coreUtils;
	m_frameCounter = 1;
	this->ResetArray();
	console->Debug( "HitCountingCompositeActor: at %p", this );
}

HitCountingCompositeActor::~HitCountingCompositeActor() {
	m_console->Debug( "~HitCountingCompositeActor: at %p", this );
}

void HitCountingCompositeActor::ResetArray() {
	for( u8 i = 0; i < HIT_THIS_MANY_TIMES; i++ ) {
		m_hitArray[ i ] = 0;
	}
}

void HitCountingCompositeActor::ShiftLeftAndStoreFrame() {
	for( u8 i = 0; i < HIT_THIS_MANY_TIMES - 1; i++ ) {
		m_hitArray[ i ] = m_hitArray[ i + 1 ];
	}
	m_hitArray[ HIT_THIS_MANY_TIMES - 1 ] = m_frameCounter;
}

void HitCountingCompositeActor::RecoverFromHit() {

	this->ShiftLeftAndStoreFrame();
	
	// have we not been hit enough times?
	if( m_hitArray[ 0 ] == 0 ) {
		// no - perform default behaviour
		RandomlyFallingCompositeActor::RecoverFromHit();
		return;
	}

	// have all hits been registered within a short time interval?
	u32 interval = m_hitArray[ HIT_THIS_MANY_TIMES - 1 ] - m_hitArray[ 0 ];
	m_console->Debug( "[HitCountingCompositeActor:RecoverFromHit]: difference %d", interval );
	if( interval <= IN_THIS_MANY_FRAMES && this->CanFallDownOrRecoverFromHit() ) {
		this->FallDown();
		return;
	}

	// default behaviour
	RandomlyFallingCompositeActor::RecoverFromHit();
}

void HitCountingCompositeActor::FallDown() {
	this->ResetArray();
	RandomlyFallingCompositeActor::FallDown();
}

void HitCountingCompositeActor::Render() {
	CompositeActor::Render();
	m_frameCounter++;
}
