#ifndef HITCOUNTING_COMPOSITEACTOR_H
#define HITCOUNTING_COMPOSITEACTOR_H

#include "FallingDownActor.h"
#include "MovingActor.h"
#include "SingleActionActor.h"
#include "ComboActor.h"
#include "ChargingActor.h"
#include "RandomlyFallingCompositeActor.h"

/*
	A composite actor that may switch to falling down when being hit
	enough times in a short time span
*/
class HitCountingCompositeActor : public RandomlyFallingCompositeActor {

	private:
	
	static const u8 HIT_THIS_MANY_TIMES = 4;
	static const u8 IN_THIS_MANY_FRAMES = 120;

	u32 m_frameCounter;
	u32 m_hitArray[ HIT_THIS_MANY_TIMES ]; // frame counts of last four hits
	CoreUtils* m_coreUtils;
	
	void ResetArray();
	void ShiftLeftAndStoreFrame();
	
	public:
	
	HitCountingCompositeActor(
		CoreUtils* coreUtils,
		Console* console,
		FallingDownActor* fallingDownActor,
		SingleActionActor* hitRecoveringActor,
		MovingActor* movingActor,
		SingleActionActor* singleHitActor,
		ComboActor* comboActor,
		SingleActionActor* healActor,
		SingleActionActor* mysticalActor,
		ChargingActor* chargingActor
		);
		
	virtual ~HitCountingCompositeActor();
	
	void RecoverFromHit();
	void FallDown();
	void Render();
};

#endif