#ifndef __MOVINGACTOR_H
#define __MOVINGACTOR_H

#include <nds.h>
#include "../core/core.h"
#include "../animators/animators.h"

/*
	An entity which can perform the following actions:
		- stand still
		- walk
		- turn sprite to the left or right when walking
		- get up from the ground
		
	IMPORTANT: Out of all possible Actors, this is the only one which
				can delete its animator
*/
class MovingActor {
	
	private:
	
	Console* m_console;
	Animator* m_animator;
	
	bool m_isStanding;
	bool m_isGettingUp;
	
	Event* m_getUpAnimationOverEvent;
	
	public:
	
	MovingActor( 
		Console* console,
		Animator* animator,
		bool isFacingRight
		);
	
	virtual ~MovingActor();
	
	Coords_F GetPosition();
	Coords_F GetCentrePosition();
	void RelocateTo( Coords_F position );
	
	void MoveBy( Coords_F deltaPosition );
	void Stand( bool resetAnimation );
	void GetUp();
		
	// true when this actor can be interrupted 
	// as an example, getting up from the ground is uninterruptible
	// (has to run to completion)
	// while walking or standing is interruptible
	bool IsInterruptible();
	void Cancel();
	
	bool IsFacingRight();
	void SetIsFacingRight( bool isFacingRight );
	void Render();
};

#endif