#include "SingleActionActor.h"

SingleActionActor::SingleActionActor( 
	CoreUtils* coreUtils,
	Console* console,
	Animator* animator,
	u8 phase,
	u8 alternatePhase
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	m_animator = animator;
	m_phase = phase;
	m_alternatePhase = alternatePhase;

	m_animationOverEvent = NULL;
	
	m_isActive = false;
}

SingleActionActor::~SingleActionActor() {

	if( NULL != m_animationOverEvent ) {
		delete m_animationOverEvent;
	}
}

void SingleActionActor::PerformAction() {
	this->PerformAction( NULL, 0.0f );
}

void SingleActionActor::PerformAction( 
	Event* animationProgressReachedEvent,
	float animationProgress
	) {
	
	if( NULL != m_animationOverEvent ) {
		delete m_animationOverEvent;
	}
	m_animationOverEvent = new Event();
	
	u8 phase = m_phase;
	if( 0 == m_coreUtils->Random( 2 ) ) {
		phase = m_alternatePhase;
	}
	
	m_animator->ChangePhase( 
		phase,
		m_animationOverEvent,
		animationProgressReachedEvent,
		animationProgress
		);
		
	m_isActive = true;
}

bool SingleActionActor::IsInterruptibleBySelf() {
	return !m_isActive;
}

bool SingleActionActor::IsInterruptibleByOther() {
	return true;
}

void SingleActionActor::Render() {

	if( !m_isActive ) {
		return;
	}

	m_animator->AdvanceFrame();

	// is the animation over?
	if( m_animationOverEvent->IsSet() ) {
		this->Cancel();
		return;
	}
	
	m_animator->Render();
}

void SingleActionActor::Cancel() {

	if( NULL != m_animationOverEvent ) {
		delete m_animationOverEvent;
		m_animationOverEvent = NULL;
		m_isActive = false;
	}
}