#ifndef __SINGLEACTION_ACTOR_H
#define __SINGLEACTION_ACTOR_H

#include <nds.h>
#include "../core/core.h"
#include "../animators/animators.h"

/*
	An entity which can perform the following actions:
		- an action whose animation will not loop, and will stop after 
		  the last frame of the phase; this action is not interruptible
		  
	This entity is not interruptible by self, and it interruptible by others
		
	IMPORTANT: This Actor does NOT delete its animator
*/
class SingleActionActor {
	
	private:
	CoreUtils* m_coreUtils;
	u8 m_phase;
	u8 m_alternatePhase;
	Animator* m_animator;
	Event* m_animationOverEvent;
	
	protected:
	
	Console* m_console;
	bool m_isActive;
	
	public:
	
	// allows the actor to randomly select between two phases 
	// when beginning to animate
	SingleActionActor( 
		CoreUtils* coreUtils,
		Console* console,
		Animator* animator,
		u8 phase,
		u8 alternatePhase
		);
	
	virtual ~SingleActionActor();
	
	void PerformAction();
	// allow an event to be raised when the action has 
	// reached a certain proportional progress
	void PerformAction( 
		Event* animationProgressReachedEvent,
		float animationProgress
		);
	
	bool IsInterruptibleBySelf();
	virtual bool IsInterruptibleByOther();
	void Cancel();
	
	void Render();
};

#endif