#include "UtilitiesCompositeActor.h"

UtilitiesCompositeActor::UtilitiesCompositeActor(
	Console* console,
	FallingDownActor* fallingDownActor,
	SingleActionActor* hitRecoveringActor,
	MovingActor* movingActor,
	SingleActionActor* singleHitActor,
	ComboActor* comboActor,
	SingleActionActor* healActor,
	SingleActionActor* mysticalActor,
	ChargingActor* chargingActor
	) : CompositeActor(
		console,
		fallingDownActor,
		hitRecoveringActor,
		movingActor,
		singleHitActor,
		comboActor,
		healActor,
		mysticalActor,
		chargingActor
		) {
		
	console->Debug( "UtilitiesCompositeActor: at %p", this );
}

UtilitiesCompositeActor::~UtilitiesCompositeActor() {
	m_console->Debug( "~UtilitiesCompositeActor: at %p", this );
}

Coords_F UtilitiesCompositeActor::GetPosition() {
	return m_movingActor->GetPosition();
}

float UtilitiesCompositeActor::DistanceTo( UtilitiesCompositeActor* other ) {
	Coords_F positionDifference = this->GetPosition() - other->GetPosition();
	return positionDifference.GetLength();
}

Coords_F UtilitiesCompositeActor::DeltaTo( UtilitiesCompositeActor* other, float speed ) {
	return this->DeltaTo( other->GetPosition(), speed );
}

Coords_F UtilitiesCompositeActor::DeltaTo( Coords_F target, float speed ) {
	Coords_F delta = target - this->GetPosition();
	delta.NormalizeAndMultiplyBy( speed );
	return delta;
}

bool UtilitiesCompositeActor::IsFacingRight() {
	return m_movingActor->IsFacingRight();
}

bool UtilitiesCompositeActor::IsFacingWithinDistanceOf( UtilitiesCompositeActor* other, float distance ) {
	return this->IsWithinDistanceOfVerticallyAdjusted( other, distance ) && 
		   this->IsFacing( other );
}

Coords_F UtilitiesCompositeActor::GetCentrePosition() {
	return m_movingActor->GetCentrePosition();
}

bool UtilitiesCompositeActor::IsWithinDistanceOfVerticallyAdjusted( 
	UtilitiesCompositeActor* other, 
	float distance 
	) {

	Coords_F me = this->GetCentrePosition();
	Coords_F him = other->GetCentrePosition();
	float distanceVertically = 0.15f * distance;
	
	if( fabs( me.x - him.x ) > distance ) {
		return false;
	}
	
	if( fabs( me.y - him.y ) > distanceVertically ) {
		return false;
	}

	return true;
}

bool UtilitiesCompositeActor::IsFacing( UtilitiesCompositeActor* other ) {
	// either I'm facing right and enemy is to my right, or
	// I'm facing left, and enemy is to my left
	if( ( !this->IsFacingRight() && other->GetPosition().x <= this->GetPosition().x ) ||
		( this->IsFacingRight() && other->GetPosition().x > this->GetPosition().x ) ) {
		
		return true;
	}

	return false;
}

void UtilitiesCompositeActor::SetIsFacingRight( bool isFacingRight ) {
	m_movingActor->SetIsFacingRight( isFacingRight );
}

bool UtilitiesCompositeActor::IsToTheLeftOf( UtilitiesCompositeActor* other ) {
	return this->GetPosition().x <= other->GetPosition().x;
}