#include "Animator.h"

Animator::Animator( 
	Console* console,
	OAMSprite* sprite,
	FrameDescriptor* frameDescriptor
	) {
	
	m_console = console;
	m_sprite = sprite;
	m_frameDescriptor = frameDescriptor;
	
	this->ChangePhase( 0 );
}

Animator::~Animator() {
	delete m_frameDescriptor;
	delete m_sprite;
}

void Animator::AdvanceFrame() {
	u8 videoFramesPerAnimationFrame = 
		m_frameDescriptor->GetPhaseVideoFramePerAnimationFrame( m_phase );
		
	u8 phaseFramesCount = m_frameDescriptor->GetPhaseFramesCount( m_phase );
	
	m_frameCountSinceLastFrameChange++;
	
	// should we raise the requested frame progress event?
	if( NULL != m_requestedFrameProgressReachedEvent ) {
		float progress = (float)m_frame / (float)phaseFramesCount;
		if( progress >= m_requestedFrameProgress ) {
			m_requestedFrameProgressReachedEvent->Set();
			// whoever gave me the event is expected to delete it
			m_requestedFrameProgressReachedEvent = NULL;
		}
	}
	
	// do we need to advance frame?
	if( m_frameCountSinceLastFrameChange > videoFramesPerAnimationFrame ) {
		
		m_frame++;
		
		m_frameCountSinceLastFrameChange = 0;
		
		if( m_frame >= phaseFramesCount ) {
			m_frame = 0;
			
			// raise event if one exists
			if( NULL != m_endOfLastFrameEvent ) {
				m_endOfLastFrameEvent->Set();
				// whoever gave me the event is expected to delete it
				m_endOfLastFrameEvent = NULL;
			}
		}
		
		// advance frame in sprite as well
		m_sprite->ChangeFrame( m_phase, m_frame );
	}
}

void Animator::Render() {
	m_sprite->Render();
}

OAMSprite* Animator::GetSprite() {
	return m_sprite;
}

void Animator::ChangePhase( u8 phase ) {
	this->ChangePhase( phase, NULL );
}

void Animator::ChangePhase( u8 phase, Event* endOfLastFrameEvent ) {
	this->ChangePhase( phase, endOfLastFrameEvent, NULL, 0.0f );
}

void Animator::ChangePhase( 
	u8 phase, 
	Event* endOfLastFrameEvent,
	Event* requestedFrameProgressReachedEvent,
	float requestedFrameProgress
	) {

	m_endOfLastFrameEvent = endOfLastFrameEvent;
	
	m_requestedFrameProgressReachedEvent = requestedFrameProgressReachedEvent;
	m_requestedFrameProgress = requestedFrameProgress;
	
	if( phase >= m_frameDescriptor->GetPhaseCount() ) {
		m_console->Error( "[Animator]ChangePhase: phase out of range: %d", phase );
	}
	
	m_phase = phase;
	m_frame = 0;
	m_sprite->ChangeFrame( m_phase, m_frame );
	
	m_frameCountSinceLastFrameChange = 0;
}
