#ifndef __ANIMATOR_H
#define __ANIMATOR_H

#include <nds.h>
#include "../core/core.h"
#include "../framedescriptors/framedescriptors.h"

/*
	An entity which can animate a sprite
*/
class Animator {
	
	private:
	
	Console* m_console;
	OAMSprite* m_sprite;
	
	FrameDescriptor* m_frameDescriptor;
	u8 m_phase;
	u8 m_frame;
	u8 m_frameCountSinceLastFrameChange;
	
	Event* m_endOfLastFrameEvent;
	
	Event* m_requestedFrameProgressReachedEvent;
	float m_requestedFrameProgress;
	
	public:
	
	Animator( 
		Console* console,
		OAMSprite* sprite,
		FrameDescriptor* frameDescriptor
		);

	virtual ~Animator();
	
	void ChangePhase( u8 phase );
	// an overload which raises an event when animation reaches end of last frame
	void ChangePhase( 
		u8 phase, 
		Event* endOfLastFrameEvent 
		);
		
	// an overload which raises an event when animation reaches end of last frame
	// and also raises an event when animation reaches a certain specified progress
	// (for example, if 0.5f is the progress, changing to third frame of a four frame
	// animation would raise the progress event).
	void ChangePhase( 
		u8 phase, 
		Event* endOfLastFrameEvent,
		Event* requestedFrameProgressReachedEvent,
		float requestedFrameProgress
		);
	
	void AdvanceFrame();
	void Render();
	
	OAMSprite* GetSprite();
};

#endif