#include "AnimatorFactory.h"

#include <man.h>
#include <smecher.h>
#include <smecherBlue.h>
#include <smecherGreen.h>
#include <smecherPurple.h>
#include <smecherRed.h>
#include <cutitar.h>
#include <mascat.h>
#include <mascatGreen.h>
#include <mascatRed.h>
#include <meiss.h>
#include <meissDark.h>
	
AnimatorFactory::AnimatorFactory( 
	Console* console,
	SpriteManager* spriteManager
	) {
	
	m_console = console;
	m_spriteManager = spriteManager;
}

AnimatorFactory::~AnimatorFactory() {
}

Animator* AnimatorFactory::Create( 
	OAMSprite* sprite,
	FrameDescriptor* frameDescriptor
	) {
	
	return new Animator( m_console, sprite, frameDescriptor );
}

Animator* AnimatorFactory::Create( AnimatorTypes type ) {

	Coords_F coords( 50.0f, 50.0f );
	FrameDescriptor* frameDescriptor;
	OAMSprite* sprite;

	switch( type ) {
	
		case ANIMATOR_PLAYER:
			
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) manTiles, 
				256, 
				(u8*) manPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new PlayerFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_SMECHER:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) smecherTiles, 
				256, 
				(u8*) smecherPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new SmecherFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
		
		case ANIMATOR_SMECHER_BLUE:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) smecherBlueTiles, 
				256, 
				(u8*) smecherBluePal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new SmecherFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_SMECHER_GREEN:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) smecherGreenTiles, 
				256, 
				(u8*) smecherGreenPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new SmecherFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_SMECHER_PURPLE:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) smecherPurpleTiles, 
				256, 
				(u8*) smecherPurplePal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new SmecherFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_SMECHER_RED:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) smecherRedTiles, 
				256, 
				(u8*) smecherRedPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new SmecherFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
		
		case ANIMATOR_CUTITAR:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) cutitarTiles, 
				256, 
				(u8*) cutitarPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new CutitarFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_MASCAT:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) mascatTiles, 
				256, 
				(u8*) mascatPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new MascatFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_MASCAT_GREEN:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) mascatGreenTiles, 
				256, 
				(u8*) mascatGreenPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new MascatFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_MASCAT_RED:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) mascatRedTiles, 
				256, 
				(u8*) mascatRedPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new MascatFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_MEISS:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) meissTiles, 
				256, 
				(u8*) meissPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new MeissFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
			
		case ANIMATOR_MEISS_DARK:
		
			sprite = m_spriteManager->AllocateMainScreenSprite( 
				(u8*) meissDarkTiles, 
				256, 
				(u8*) meissDarkPal
				);
			sprite->MoveTo( coords );
			frameDescriptor = new MeissFrameDescriptor( m_console );
			return new Animator( m_console, sprite, frameDescriptor );
	}
	
	m_console->Error( "[AnimatorFactory]Create: Invalid animator type!" );
	return NULL;
}
