#include "StageBackground.h"

StageBackground::StageBackground( 
	Console* console,
	u8 chunkCount,
	FuncBool* canScroll
	) {
	
	m_console = console;
	m_chunkCount = chunkCount;
	m_canScroll = canScroll;
	
	m_chunks = new u16* [ m_chunkCount ];
	for( u8 i = 0; i < m_chunkCount; i++ ) {
		m_chunks[ i ] = NULL;
	}
	
	m_scrollingBackground = NULL;
}

StageBackground::~StageBackground() {
	delete m_chunks;
	delete m_scrollingBackground;
	m_console->Debug( "~StageBackground: at %p", this );
}

void StageBackground::InitializeChunk(
	u8 which,
	u16* chunk
	) {
	
	m_chunks[ which ] = chunk;
}

void StageBackground::Initialize(
	u32* backgroundTileData,
	u32 backgroundTileDataLength,
	u16* backgroundPaletteData,
	u32 backgroundPaletteDataLength
	) {
	
	if( NULL != m_scrollingBackground ) {
		return;
	}
	
	for( u8 i = 0; i < m_chunkCount; i++ ) {
		if( NULL == m_chunks[ i ] ) {
			m_console->Error( "Cannot initialize background if not all chunks are initialized" );
			return;
		}
	}
	
	m_scrollingBackground = new ScrollingBackground(
		m_console,
		backgroundTileData,
		backgroundTileDataLength,
		backgroundPaletteData,
		backgroundPaletteDataLength,
		m_chunks,
		m_chunkCount
		);
		
	m_scrollX = m_scrollingBackground->GetScrollAmount();
}

void StageBackground::ScrollBy( float amount ) {

	if( !this->CanScroll() ) {
		return;
	}

	// wrap the integer-based scrolling background to provide 
	// floating point-based scrolling
	m_scrollX = m_scrollX + amount;
	
	u16 currentIntegerAmount = m_scrollingBackground->GetScrollAmount();
	u16 newIntegerAmount = (u16)m_scrollX;
	
	if( newIntegerAmount > currentIntegerAmount ) {
		m_scrollingBackground->ScrollBy( newIntegerAmount - currentIntegerAmount );
	}
}

bool StageBackground::IsAtEnd() {
	return m_scrollingBackground->IsAtEnd();
}

bool StageBackground::CanScroll() {
	return m_canScroll->Invoke() && !m_scrollingBackground->IsAtEnd();
}

u16 StageBackground::GetCurrentScrollAmount() {
	return m_scrollingBackground->GetScrollAmount();
}

void StageBackground::Render() {
	m_scrollingBackground->Render();
}

void StageBackground::IncreaseMosaicEffect() {
	m_scrollingBackground->IncreaseMosaicEffect();
}
void StageBackground::ResetMosaicEffect() {
	m_scrollingBackground->ResetMosaicEffect();
}