#include "Character.h"

Character::Character(
	Soundboard* soundBoard,
	UtilitiesCompositeActor* actor,
	List<Character>* iAmInThisPool,
	CharacterProperties properties,
	CharacterTypes type,
	char* name
	) {
	
	m_soundBoard = soundBoard;
	m_actor = actor;
	m_iAmInThisPool = iAmInThisPool;
	m_type = type;
	m_lastVictim = NULL;
	m_lastAggressor = NULL;
	m_properties = properties;
	
	// copy name
	memmove( m_name, name, MAX_NAME_LENGTH );
}

Character::~Character() {

	// if I am in a pool, remove myself from it
	if( NULL != m_iAmInThisPool ) {
		m_iAmInThisPool->DeleteWhere( this );
	}
	
	delete m_actor;
}
	
UtilitiesCompositeActor* Character::GetActor() {
	return m_actor;
}

CharacterProperties* Character::GetProperties() {
	return &m_properties;
}

CharacterTypes Character::GetType() {
	return m_type;
}

Character* Character::GetLastVictim() {
	return m_lastVictim;
}

void Character::SetLastVictim( Character* character ) {
	m_lastVictim = character;
}

Character* Character::GetLastAggressor() {
	return m_lastAggressor;
}

void Character::SetLastAggressor( Character* character ) {
	m_lastAggressor = character;
}

char* Character::GetName() {
	return m_name;
}

Soundboard* Character::GetSoundboard() {
	return m_soundBoard;
}