#ifndef __CHARACTER_H
#define __CHARACTER_H

#include "../core/core.h"
#include "../actors/actors.h"
#include "../soundboards/soundboards.h"
#include "CharacterProperties.h"
#include "CharacterTypes.h"

/*
	Represents an entity with various characteristics, and an actor
	
	NOTE: Deletes actors
*/
class Character{
	
	private:
	
	Soundboard* m_soundBoard;
	UtilitiesCompositeActor* m_actor;
	List<Character>* m_iAmInThisPool;
	CharacterProperties m_properties;
	CharacterTypes m_type;
	
	static const u8 MAX_NAME_LENGTH = 128;
	char m_name[ MAX_NAME_LENGTH ];
	
	Character* m_lastVictim; // my last victim
	Character* m_lastAggressor; // my last aggressor
	
	public:
	
	Character(
		Soundboard* soundBoard,
		UtilitiesCompositeActor* actor,
		List<Character>* iAmInThisPool,
		CharacterProperties properties,
		CharacterTypes type,
		char* name
		);

	virtual ~Character();
	
	UtilitiesCompositeActor* GetActor();
	CharacterProperties* GetProperties();
	CharacterTypes GetType();
	char* GetName();
	Soundboard* GetSoundboard();
	Character* GetLastVictim();
	void SetLastVictim( Character* character );
	Character* GetLastAggressor();
	void SetLastAggressor( Character* character );
};

#endif