#include "CharacterFactory.h"

CharacterFactory::CharacterFactory(
	Console* console,
	SoundboardFactory* soundboardFactory
	) {
	
	m_console = console;
	m_soundboardFactory = soundboardFactory;
	m_nonPlayerCharacters = new List<Character>();
	
	InitPlayerProperties();
	InitSmecherProperties();
	InitMeissProperties();
	InitCutitarProperties();
	InitMascatProperties();
}

CharacterFactory::~CharacterFactory() {
	delete m_nonPlayerCharacters;
}

void CharacterFactory::InitPlayerProperties() {
	m_playerProperties.SetMaxLife( m_initialPlayerStats.GetInitialLife() );
	m_playerProperties.SetMaxEnergy( 1000 );
	m_playerProperties.SetEnergy( m_initialPlayerStats.GetInitialEnergy() );
	m_playerProperties.SetSingleHitDamage( 600 );
	m_playerProperties.SetComboHitDamage( 50 );
	m_playerProperties.SetChargingDamage( 0 );
	m_playerProperties.SetWalkingSpeed( 1.5f );
	m_playerProperties.SetChargingSpeed( 2.0f );
	
	m_playerProperties.SetChanceToKeepWalking( 993 );
	m_playerProperties.SetChanceToKeepNonWalking( 980 );
	m_playerProperties.SetChanceToAttack( 10 );
	m_playerProperties.SetChanceToCharge( 3 );
}

void CharacterFactory::InitSmecherProperties() {
	m_smecherWeakProperties = m_playerProperties;
	m_smecherWeakProperties.SetMaxLife( 500 );
	m_smecherWeakProperties.SetWalkingSpeed( 0.5f );
	m_smecherWeakProperties.SetSingleHitDamage( 60 );
	m_smecherWeakProperties.SetChanceToKeepWalking( 995 );
	
	m_smecherStrongProperties = m_smecherWeakProperties;
	m_smecherStrongProperties.SetMaxLife( 600 );
	m_smecherStrongProperties.SetSingleHitDamage( 72 );
	m_smecherStrongProperties.SetChanceToKeepWalking( 997 );
	m_smecherStrongProperties.SetWalkingSpeed( 0.625f );
	
	m_smecherBossProperties = m_smecherWeakProperties;
	m_smecherBossProperties.SetMaxLife( 2400 );
	m_smecherBossProperties.SetSingleHitDamage( 100 );
	m_smecherBossProperties.SetWalkingSpeed( 1.0f );
	m_smecherBossProperties.SetChanceToAttack( 1000 );
	m_smecherBossProperties.SetChanceToKeepWalking( 998 );
}

void CharacterFactory::InitMeissProperties() {
	
	m_meissNonchargingProperties = m_smecherStrongProperties;
	m_meissNonchargingProperties.SetMaxLife( 800 );
	m_meissNonchargingProperties.SetSingleHitDamage( 200 );
	m_meissNonchargingProperties.SetWalkingSpeed( 0.4f );
	m_meissNonchargingProperties.SetChanceToAttack( 20 );
	
	m_meissProperties = m_meissNonchargingProperties;
	m_meissProperties.SetMaxLife( 900 );
	m_meissProperties.SetSingleHitDamage( 250 );
	m_meissProperties.SetWalkingSpeed( 0.45f );
	m_meissProperties.SetChargingSpeed( 1.35f );
	m_meissProperties.SetChargingDamage( 100 );
	
	m_meissBossProperties = m_meissProperties;
	m_meissBossProperties.SetSingleHitDamage( 800 );
	m_meissBossProperties.SetChanceToKeepWalking( 990 );
	m_meissBossProperties.SetWalkingSpeed( 0.9f );
	m_meissBossProperties.SetMaxLife( 3600 );
	m_meissBossProperties.SetChargingSpeed( 2.5f );
	m_meissBossProperties.SetChanceToAttack( 1000 );
	m_meissBossProperties.SetChanceToCharge( 60 );
	m_meissBossProperties.SetChargingDamage( 500 );
}

void CharacterFactory::InitCutitarProperties() {
	m_cutitarProperties = m_meissProperties;
	m_cutitarProperties.SetWalkingSpeed( 1.2f );
	m_cutitarProperties.SetSingleHitDamage( 60 );
	m_cutitarProperties.SetChanceToAttack( 30 );
	m_cutitarProperties.SetMaxLife( 500 );
	
	m_cutitarBossProperties = m_cutitarProperties;
	m_cutitarBossProperties.SetMaxLife( 2000 );
	m_cutitarBossProperties.SetWalkingSpeed( 1.3f );
	m_cutitarBossProperties.SetChanceToAttack( 300 );
	m_cutitarBossProperties.SetChanceToCharge( 60 );
	m_cutitarBossProperties.SetChargingSpeed( 2.15f );
	m_cutitarBossProperties.SetChargingDamage( 300 );
	m_cutitarBossProperties.SetSingleHitDamage( 40 );
}

void CharacterFactory::InitMascatProperties() {
	m_mascatWeakProperties = m_cutitarProperties;
	m_mascatWeakProperties.SetWalkingSpeed( 0.8f );
	m_mascatWeakProperties.SetSingleHitDamage( 140 );
	m_mascatWeakProperties.SetChanceToAttack( 25 );
	m_mascatWeakProperties.SetMaxLife( 800 );
	
	m_mascatStrongProperties = m_mascatWeakProperties;
	m_mascatStrongProperties.SetChanceToCharge( 5 );
	m_mascatStrongProperties.SetChargingDamage( 160 );
	m_mascatStrongProperties.SetChargingSpeed( 1.35f );
	
	m_mascatBossProperties = m_mascatStrongProperties;
	m_mascatBossProperties.SetChanceToAttack( 1000 );
	m_mascatBossProperties.SetChargingSpeed( 1.15f );
	m_mascatBossProperties.SetMaxLife( 1700 );
	m_mascatBossProperties.SetWalkingSpeed( 1.2f );
	m_mascatBossProperties.SetSingleHitDamage( 200 );
}


Character* CharacterFactory::Create(
	UtilitiesCompositeActor* actor,
	CharacterTypes type,
	s16 initialLife,
	s16 initialEnergy
	) {
	
	Character* result = this->Helper_Create( actor, type );
	result->GetProperties()->SetLife( initialLife );
	result->GetProperties()->SetEnergy( initialEnergy );
	return result;
}

Character* CharacterFactory::Create(
	UtilitiesCompositeActor* actor,
	CharacterTypes type
	) {
	
	Character* result = this->Helper_Create( actor, type );
	return result;
}

Character* CharacterFactory::Helper_Create(
	UtilitiesCompositeActor* actor,
	CharacterTypes type
	) {
	
	Character* character;
	Soundboard* soundboard = MakeSoundboard( type );
	
	switch( type ) {
		case PLAYER_CHARACTER:
			character = new Character( 
				soundboard,
				actor,
				NULL,
				m_playerProperties,
				type,
				(char*)"PLAYER"
				);
			return character;
			
		case SMECHER_CHARACTER:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_smecherWeakProperties,
				type,
				(char*)"STEVE RYAN"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case SMECHER_CHARACTER_BLUE:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_smecherStrongProperties,
				type,
				(char*)"HASSEL"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case SMECHER_CHARACTER_GREEN:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_smecherWeakProperties,
				type,
				(char*)"SHELDON COWINS"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case SMECHER_CHARACTER_PURPLE:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_smecherStrongProperties,
				type,
				(char*)"DUG"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case SMECHER_CHARACTER_RED:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_smecherBossProperties,
				type,
				(char*)"FOREMAN SERAM"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
			
		case CUTITAR_CHARACTER:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_cutitarProperties,
				type,
				(char*)"SHADY JOE"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case CUTITAR_CHARACTER_BOSS:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_cutitarBossProperties,
				type,
				(char*)"IAHIM THE UNSTABLE"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
			
		case MASCAT_CHARACTER:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_mascatWeakProperties,
				type,
				(char*)"SMITH"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case MASCAT_CHARACTER_GREEN:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_mascatStrongProperties,
				type,
				(char*)"NAIT SABES"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case MASCAT_CHARACTER_RED:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_mascatBossProperties,
				type,
				(char*)"ARCHITECT"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
			
		case MEISS_CHARACTER:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_meissProperties,
				type,
				(char*)"MEISS"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case MEISS_CHARACTER_DARK:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_meissNonchargingProperties,
				type,
				(char*)"HEAVY JACKSON"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
		case MEISS_CHARACTER_BOSS:
			character = new Character(
				soundboard,
				actor,
				m_nonPlayerCharacters,
				m_meissBossProperties,
				type,
				(char*)"MIKE RANDO"
				);
			m_nonPlayerCharacters->Add( character );
			return character;
	}
	
	m_console->Error( "[CharacterFactory::Create]: Unrecognized character type" );
	return NULL;
}

Soundboard* CharacterFactory::MakeSoundboard( 
	CharacterTypes type	
	) {
	
	switch( type ) {
		case PLAYER_CHARACTER:
			return m_soundboardFactory->Get( PLAYER_SOUNDBOARD );
			
		case SMECHER_CHARACTER:
		case SMECHER_CHARACTER_BLUE:
		case SMECHER_CHARACTER_GREEN:
		case SMECHER_CHARACTER_PURPLE:
		case SMECHER_CHARACTER_RED:
			return m_soundboardFactory->Get( SMECHER_SOUNDBOARD );
			
		case CUTITAR_CHARACTER:
		case CUTITAR_CHARACTER_BOSS:
			return m_soundboardFactory->Get( CUTITAR_SOUNDBOARD );
			
		case MASCAT_CHARACTER:
		case MASCAT_CHARACTER_GREEN:
		case MASCAT_CHARACTER_RED:
			return m_soundboardFactory->Get( MASCAT_SOUNDBOARD );
			
		case MEISS_CHARACTER:
		case MEISS_CHARACTER_DARK:
		case MEISS_CHARACTER_BOSS:
			return m_soundboardFactory->Get( MEISS_SOUNDBOARD );
	}
	
	m_console->Error( "[CharacterFactory::MakeSoundboard]: Unrecognized character type" );
	return NULL;
}

List<Character>* CharacterFactory::GetNonPlayerCharacterPool() {
	return m_nonPlayerCharacters;
}