#ifndef __CHARACTER_FACTORY_H
#define __CHARACTER_FACTORY_H

#include "../core/core.h"
#include "../actors/actors.h"
#include "../soundboards/soundboards.h"
#include "CharacterTypes.h"
#include "Character.h"
#include "CharacterProperties.h"

/*
	Makes characters
*/
class CharacterFactory {
	
	private:
	
	CharacterProperties m_playerProperties;
	
	CharacterProperties m_smecherWeakProperties;
	CharacterProperties m_smecherStrongProperties;
	CharacterProperties m_smecherBossProperties;
	
	CharacterProperties m_mascatWeakProperties;
	CharacterProperties m_mascatStrongProperties;
	CharacterProperties m_mascatBossProperties;
	
	CharacterProperties m_meissProperties;
	CharacterProperties m_meissNonchargingProperties;
	CharacterProperties m_meissBossProperties;
	
	CharacterProperties m_cutitarProperties;
	CharacterProperties m_cutitarBossProperties;
	
	Console* m_console;
	SoundboardFactory* m_soundboardFactory;
	
	List<Character>* m_nonPlayerCharacters;
	InitialPlayerStats m_initialPlayerStats;
	
	void InitPlayerProperties();
	void InitSmecherProperties();
	void InitCutitarProperties();
	void InitMascatProperties();
	void InitMeissProperties();
	
	Soundboard* MakeSoundboard( CharacterTypes type );	
	Character* Helper_Create(
		UtilitiesCompositeActor* actor,
		CharacterTypes type
		);
	
	public:
	
	CharacterFactory(
		Console* console,
		SoundboardFactory* soundboardFactory
		);

	virtual ~CharacterFactory();
	
	Character* Create(
		UtilitiesCompositeActor* actor,
		CharacterTypes type
		);
		
	Character* Create(
		UtilitiesCompositeActor* actor,
		CharacterTypes type,
		s16 initialLife,
		s16 initialEnergy
		);
		
	List<Character>* GetNonPlayerCharacterPool();
};

#endif