#ifndef __CHARACTER__PROPERTIES_H
#define __CHARACTER__PROPERTIES_H

#include "../core/core.h"

struct InitialPlayerStats {
	public:
	s16 GetInitialLife() {
		return 1000;
	}
	s16 GetInitialEnergy() {
		return 0;
	}
	FrameRunnerResult ToFrameRunnerResult() {
		return FrameRunnerResult( 
			this->GetInitialLife(),
			this->GetInitialEnergy()
			);
	}
};

struct CharacterProperties {

	private:
	
	static const s16 POWER_STRIKE_ENERGY_COST = 120;
	static const s16 HEAL_AND_MYSTICAL_ENERGY_COST = 1000;
	static const s16 HEAL_AMOUNT = 500;
	static const s16 MYSTICAL_DAMAGE = 400;
	
	s16 m_maxLife;
	s16 m_life;
	s16 m_singleHitDamage;
	s16 m_comboHitDamage;
	s16 m_chargingDamage;
	float m_walkingSpeed;
	float m_chargingSpeed;
	s16 m_maxEnergy;
	s16 m_energy;
	
	s16 m_chanceToAttack;
	s16 m_chanceToKeepWalking;
	s16 m_chanceToKeepNonWalking;
	s16 m_chanceToCharge;
	
	public: 
	
	s16 GetLife() { return m_life; }
	void DecreaseLifeBy( s16 val ) {
		m_life = m_life - val;
		if( m_life < 0 ) {
			m_life = 0;
		}
	}
	void HealBy( s16 val ) {
		m_life = m_life + val;
		if( m_life > m_maxLife ) {
			m_life = m_maxLife;
		}
	}
	void HealAndSpendEnergy() {
		if( this->GetEnergy() < this->GetHealAndMysticalEnergyCost() ) {
			return;
		}
		
		this->DecreaseEnergyBy(
			this->GetHealAndMysticalEnergyCost()
			);
		this->HealBy( HEAL_AMOUNT );
	}
	void SetLife( s16 val ) {
		m_life = val;
	}
	
	void SetMaxLife( s16 val ) { 
		m_maxLife = val; 
		m_life = m_maxLife;
	}
	s16 GetMaxLife() { return m_maxLife; }
	
	s16 GetEnergy() { return m_energy; }
	void DecreaseEnergyBy( s16 val ) {
		m_energy = m_energy - val;
		if( m_energy < 0 ) {
			m_energy = 0;
		}
	}
	void GainEnergy( s16 val ) {
		m_energy = m_energy + val;
		if( m_energy > m_maxEnergy ) {
			m_energy = m_maxEnergy;
		}
	}
	void SetEnergy( s16 val ) {
		m_energy = val;
	}
	s16 GetPowerStrikeEnergyCost() {
		return POWER_STRIKE_ENERGY_COST;
	}
	s16 GetHealAndMysticalEnergyCost() {
		return HEAL_AND_MYSTICAL_ENERGY_COST;
	}
	
	void SetMaxEnergy( s16 val ) { 
		m_maxEnergy = val; 
		m_energy = 0;
	}
	s16 GetMaxEnergy() { return m_maxEnergy; }
	
	s16 GetMysticalDamage() {
		return MYSTICAL_DAMAGE;
	}
	
	void SetSingleHitDamage( s16 val ) { m_singleHitDamage = val; }
	s16 GetSingleHitDamage() { return m_singleHitDamage; }
	
	void SetComboHitDamage( s16 val ) { m_comboHitDamage = val; }
	s16 GetComboHitDamage() { return m_comboHitDamage; }
	
	void SetChargingDamage( s16 val ) { m_chargingDamage = val; }
	s16 GetChargingDamage() { return m_chargingDamage; }
	
	void SetWalkingSpeed( float val ) { m_walkingSpeed = val; }
	float GetWalkingSpeed() { return m_walkingSpeed; }
	
	void SetChargingSpeed( float val ) { m_chargingSpeed = val; }
	float GetChargingSpeed() { return m_chargingSpeed; }
	
	void SetChanceToAttack( s16 val ) { m_chanceToAttack = val; }
	s16 GetChanceToAttack() { return m_chanceToAttack; }
	
	void SetChanceToKeepWalking( s16 val ) { m_chanceToKeepWalking = val; }
	s16 GetChanceToKeepWalking() { return m_chanceToKeepWalking; }
	
	void SetChanceToKeepNonWalking( s16 val ) { m_chanceToKeepNonWalking = val; }
	s16 GetChanceToKeepNonWalking() { return m_chanceToKeepNonWalking; }
	
	void SetChanceToCharge( s16 val ) { m_chanceToCharge = val; }
	s16 GetChanceToCharge() { return m_chanceToCharge; }
};

#endif