#include "ControllerFactory.h"

ControllerFactory::ControllerFactory(
	Console* console,
	AudioPlayer* audioPlayer,
	LevelFactory* levelFactory
	) {
	m_console = console;
	m_audioPlayer = audioPlayer;
	m_levelFactory = levelFactory;
}

ControllerFactory::~ControllerFactory() {
}

Controller* ControllerFactory::Create( 
	ControllerTypes type,
	FrameRunnerResult initialConditions
	) {
	switch( type ) {
		case MENU_CONTROLLER:
			return new MenuController( m_console, m_audioPlayer, m_levelFactory, initialConditions );
		case ROMANIA_CONTROLLER:
			return new RomaniaController( m_audioPlayer, m_levelFactory, initialConditions );
		case USA_CONTROLLER:
			return new UsaController( m_audioPlayer, m_levelFactory, initialConditions );
		case CHINA_CONTROLLER:
			return new ChinaController( m_audioPlayer, m_levelFactory, initialConditions );
		case BRITAIN_CONTROLLER:
			return new BritainController( m_audioPlayer, m_levelFactory, initialConditions );
		case CORE_CONTROLLER:
			return new CoreController( m_audioPlayer, m_levelFactory, initialConditions );
	}
	
	m_console->Error( "[ControllerFactory::Create]: Unrecognized controller type" );
	return NULL;
}