#include "MenuController.h"

MenuController::MenuController(
	Console* console,
	AudioPlayer* audioPlayer,
	LevelFactory* levelFactory,
	FrameRunnerResult initialConditions
) : Controller( audioPlayer, levelFactory, initialConditions ) {
	
	m_console = console;
}

MenuController::~MenuController() {
}

FrameRunnerResult MenuController::Run() {
	
	PrintTips();
	
	FrameRunner* level = m_levelFactory->CreateTransition( MAIN_MENU );
	Song* song = m_audioPlayer->LoadSong( MOD_MENU );
	song->Play();
	
	level->Start();
	
	song->Stop();
	delete song;
	delete level;
	
	m_console->Clear();
	
	// since we're leaving the menu, the result is simply the overall
	// starting state
	return m_initialPlayerStats.ToFrameRunnerResult();
}

void MenuController::PrintTips() {
	u8 titleRow = 1;
	m_console->PrintAt( 10, titleRow, "PLAYING TIPS" );
	m_console->PrintAt( 1, titleRow + 2, "* COMPLETE COMBOS TO EARN" );
	m_console->PrintAt( 1, titleRow + 3, "  ENERGY BONUSES" );
	m_console->PrintAt( 1, titleRow + 5, "* SPEND ENERGY TO DAMAGE YOUR" );
	m_console->PrintAt( 1, titleRow + 6, "  ENEMIES OR TO HEAL YOURSELF" );
	m_console->PrintAt( 1, titleRow + 8, "* YOU HAVE ONE LIFE AND NO" );
	m_console->PrintAt( 1, titleRow + 9, "  CONTINUES - HEAL IF NEEDED" );
	
	u8 creditsRow = 15;
	m_console->PrintAt( 1, creditsRow,     "Program        by Sebastian M." );
	m_console->PrintAt( 1, creditsRow + 1, "Design         by Sebastian M." );
	m_console->PrintAt( 1, creditsRow + 2, "Story          by Sebastian M." );
	m_console->PrintAt( 1, creditsRow + 3, "Graphics       by Sebastian M." );
	m_console->PrintAt( 1, creditsRow + 4, "Music          by Sebastian M." );
	m_console->PrintAt( 1, creditsRow + 5, "Sound effects  by Sebastian M." );
	
	u8 thanksRow = 22;
	m_console->PrintAt( 5, thanksRow, "Thank you for playing!" );
}