#include <maxmod9.h>
#include "Song.h"
#include "AudioPlayer.h"

#include "soundbank_bin.h"

class Song;

AudioPlayer::AudioPlayer( 
	Console* console,
	CoreUtils* coreUtils
	) {
	
	m_console = console;
	m_coreUtils = coreUtils;
	
	m_song = NULL;
	mmInitDefaultMem( (mm_addr)soundbank_bin );
}

AudioPlayer::~AudioPlayer() {
	if( NULL != m_song ) {
		delete m_song;
	}
}

Song* AudioPlayer::LoadSong( u32 songId ) {
	
	if( NULL != m_song ) {
		m_console->Error( "LoadSong: Already loaded!" );
		return m_song;
	}
	
	m_song = new Song( songId, m_console, m_coreUtils, this );
	
	m_console->Debug( "LoadSong: %d at %p", songId, m_song );
	
	return m_song;
}

void AudioPlayer::FreeSong( Song* song ) {
	
	if( NULL == m_song ) {
		m_console->Error( "FreeSong: Nothing loaded!" );
	}
	
	if( song != m_song ) {
		m_console->Error( "FreeSong: Not found!" );
	}
	
	m_console->Debug( "FreeSong: %p", m_song );
	
	m_song = NULL;
}

SoundEffect* AudioPlayer::LoadSoundEffect( 
	u32 soundEffectId, 
	u8 multiplexingLimit
	) {
	
	return new SoundEffect(
		soundEffectId,
		multiplexingLimit,
		m_console
		);
}

SoundEffect* AudioPlayer::LoadSoundEffect( 
	u32 soundEffectId
	) {
	
	return LoadSoundEffect( soundEffectId, 1 );
}