#include <stdio.h>
#include <stdarg.h>
#include "Console.h"
#include "CoreUtils.h"
#include "CustomFont.h"

Console::Console( 
	CoreUtils* coreUtils,
	bool isDebugEnabled 
	) {
	
	m_isDebugEnabled = isDebugEnabled;
	m_coreUtils = coreUtils;
	
	m_customFont = new CustomFont();
	m_customFont->Initialize();
}

Console::~Console() {
	delete m_customFont;
}

void Console::Error( const char * format, ... ) {

	va_list arglist;
	va_start( arglist, format );
	viprintf( format, arglist );
	va_end( arglist );
	
	iprintf( "\n" );
	iprintf( "ERROR ENCOUNTERED. HALTING CPU." );
	while( 1 ) { }
}

void Console::Debug( const char * format, ... ) {

	if( !m_isDebugEnabled ) {
		return;
	}
	
	// pass through variable arguments list to a STDIO function
	va_list arglist;
	va_start( arglist, format );
	viprintf( format, arglist );
	va_end( arglist );
	
	iprintf( "\n" );
}

void Console::Print( const char * format, ... ) {

	va_list arglist;
	va_start( arglist, format );
	viprintf( format, arglist );
	va_end( arglist );
}

void Console::PrintAt( u8 column, u8 row, const char * format, ... ) {

	this->MoveCursorTo( column, row );

	va_list arglist;
	va_start( arglist, format );
	viprintf( format, arglist );
	va_end( arglist );
}

void Console::PadBlankLine() {
	iprintf( "                                " );
}

void Console::MoveCursorTo( u8 column, u8 row ) {
	iprintf( "\x1b[%d;%dH", row, column );
}

void Console::Clear() {
	iprintf("\x1b[2J");
}