#ifndef __DATATYPES_H
#define __DATATYPES_H

#include <math.h>

struct Coords_F {
	float x;
	float y;

	Coords_F() {
	}
	
	Coords_F( float xx, float yy ) {
		x = xx;
		y = yy;
	}
	
	Coords_F( const Coords_F &p ) {
		x = p.x;
		y = p.y;
	}
	
	Coords_F operator+( const Coords_F &p ) const { 
		return Coords_F( x + p.x, y + p.y );
	}
	
	Coords_F operator-( const Coords_F &p ) const { 
		return Coords_F( x - p.x, y - p.y ); 
	}
	
	bool operator!=( const Coords_F &p ) const { 
		return y != p.y || x != p.x;
	}
	
	/*
		This is based on Quake 3's fast inverse square root implementation
	*/
	float SqrtApproximation( float number ) {
		long i;
		float x2, yy;
		const float threeHalves = 1.5f;

		// some weird stuff is happening around 0
		if( number >= -0.01f && number <= 0.01f ) {
			return 0.0f;
		}
		
		x2 = number * 0.5f;
		yy  = number;
		
		// go through void to get rid of pesky dereferencing warnings, since
		// void, float, and int are all 4-byte types
		void* py = &yy; 
		i  = * ( long * ) py; //i  = * ( long * ) &yy;
		i  = 0x5f3759df - ( i >> 1 ); // crazy magic
		void* pi = &i;
		yy  = * ( float * ) pi; //yy  = * ( float * ) &i;
		
		// three iterations of Newton-Raphson
		yy  = yy * ( threeHalves - ( x2 * yy * yy ) );
		yy  = yy * ( threeHalves - ( x2 * yy * yy ) );
		yy  = yy * ( threeHalves - ( x2 * yy * yy ) );

		return 1.0f / yy;
	}
	
	void NormalizeAndMultiplyBy( float amount ) {
		float length = GetLength();
		x = x * amount / length;
		y = y * amount / length;
	}
	
	float GetLength() {
		float length = SqrtApproximation( x * x + y * y );
		return length;
	}
};


struct FrameRunnerResult {
	s16 first;
	s16 second;

	FrameRunnerResult() {
	}
	
	FrameRunnerResult( s16 f, s16 s ) {
		first = f;
		second = s;
	}
	
	FrameRunnerResult( const FrameRunnerResult &p ) {
		first = p.first;
		second = p.second;
	}
};

#endif