#ifndef __LIST__H
#define __LIST__H

#include <nds.h>

/*
	A node class for our list
*/
template <class T> class Node {
    
	private:
	
	T* m_data;
    Node* m_next;

	public:
	
	Node( T* data );
	virtual ~Node();
	
	T* GetData();
	Node* GetNext();
	void SetNext( Node* node );
};

/*
	A singly linked list
*/
template <class T> class List {
    
	private:
	
	Node<T>* m_head;
	Node<T>* m_tail;

	public:

    List();
    virtual ~List();

    void Add( T* data );
	Node<T>* GetHead();
	void DeleteWhere( T* data );
};





// ===== Implementation below, because C++ templates are finicky :( =====

template <class T> Node<T>::Node( T* data ) {
	m_data = data;
	m_next = NULL;
}

template <class T> Node<T>::~Node() {
}

template <class T> T* Node<T>::GetData() {
	return m_data;
}

template <class T> void Node<T>::SetNext( Node* node ) {
	m_next = node;
}

template <class T> Node<T>* Node<T>::GetNext() {
	return m_next;
}




template <class T> List<T>::List() {
	m_head = NULL;
	m_tail = NULL;
}

template <class T> List<T>::~List() {
	if( NULL == m_head ) {
		return;
	}
	
	Node<T>* node = m_head;
	while( NULL != node ) {
		Node<T>* next = node->GetNext();
		delete node;
		node = next;
	}
}

template <class T> void List<T>::Add( T* data ) {
	
	Node<T>* temp = new Node<T>( data );

	if( NULL == m_head) {
		m_head = temp;
	} else {
		m_tail->SetNext( temp );
	}
	m_tail = temp;
}

template <class T> void List<T>::DeleteWhere( T* data ) {
	
	if( NULL == m_head ) {
		return;
	}
	
	// are we deleting the head?
	if( m_head->GetData() == data ) {
		Node<T>* oldHead = m_head;
		m_head = m_head->GetNext();
		delete oldHead;
		return;
	}
	
	Node<T>* previous = m_head;
	
	while( NULL != previous ) {
		Node<T>* node = previous->GetNext();
		
		// pointer comparison
		if( node->GetData() == data ) {
			previous->SetNext( node->GetNext() );
			delete node;
			
			// did we delete tail?
			if( NULL == previous->GetNext() ) {
				m_tail = previous;
			}
			break;
		}	
		previous = previous->GetNext();
	}
	
}

template <class T> Node<T>* List<T>::GetHead() {
	return m_head;
}

#endif