#include "OAMSprite.h"
#include "MainScreenOAMSprite.h"
#include "MainScreenSpriteManager.h"


MainScreenSpriteManager::MainScreenSpriteManager( Console* console ) {

	m_console = console;
	
	m_spritePool = new PrioritizingSpritePool( 
		m_console, 
		MAX_SPRITES
		);
	
	m_console->Debug( "MainScreenSpriteManager: %p", this );
}
	
MainScreenSpriteManager::~MainScreenSpriteManager() {
	delete m_spritePool;
	m_console->Debug( "~MainScreenSpriteManager: %p", this );
}
	
OAMSprite* MainScreenSpriteManager::AllocateSprite( 
	u8* graphicsData, 
	u16 spriteSheetWidthInPixels,
	u8* paletteData
	) {
	
	u8 slot = m_spritePool->FindAvailableSlot();
	
	OAMSprite* newSprite = new MainScreenOAMSprite( 
		slot, 
		graphicsData, 
		spriteSheetWidthInPixels,
		m_spritePool,
		m_console,
		paletteData
		);
	
	m_spritePool->AddSprite( newSprite );
		
	m_console->Debug( 
		"[MainScreenSpriteManager] AllocateSprite: %p", 
		newSprite
		);
	
	return newSprite;
}

void MainScreenSpriteManager::ReprioritizeSprites() {
	m_spritePool->Reprioritize();
}