#ifndef OAMSPRITE_H
#define OAMSPRITE_H

// forward definition to avoid cyclic includes
class SpritePool;

#include <nds.h>
#include "SpritePool.h"
#include "Console.h"
#include "DataTypes.h"

/*
	This class is in charge of bridging sprite sheets generated from 
	image files (such as .PNG, etc.) via GRIT to the NDS hardware
	sprites and sprite/palette memory.
*/
class OAMSprite {

	private:
	
	static const u8 SPRITE_FRAME_SIDE_LENGTH = 64; // we're managing 64x64 pixel sprites
	static const float LEFT_BOUND = -64.0f;
	static const float RIGHT_BOUND = 256.0f;
	static const float TOP_BOUND = 90.0f;
	static const float BOTTOM_BOUND = 128.0f;
	static const SpriteSize m_spriteSizeEnum = SpriteSize_64x64;
	
	static const u16 SPRITE_FRAME_SIDE_LENGTH_SQUARED = 
		SPRITE_FRAME_SIDE_LENGTH * SPRITE_FRAME_SIDE_LENGTH; // shave off a few multiplications
	
	SpritePool* m_spritePool; // a reference to the sprite pool

	u8 m_priority;
	u8 m_OAMSpriteIndex; // sprite index as far as NDS is concerned
	u16* m_spriteCurrentFrameGraphicsData; // pointer to start address of current frame in the OAM
	u8* m_spriteAllGraphicsData; // pointer to array generated by GRIT to hold the tiles for this sprite
	u8 m_spriteSheetWidthInFrames; // total width of the sheet in frames (each frame is 64x64 pixels)
	u8 m_OAMExtendedPaletteIndex; // palette index as far as NDS is concerned
	OamState* m_myOAM;
	
	Coords_F m_position;
	Coords_F m_oldPosition;
	bool m_isFacingRight;
	
	void ForceOffScreen();
	void WriteToOAM(
		u8 index,
		s16 x,
		s16 y,
		u8 paletteIndex
		);
	
	protected:
		
	Console* m_console;
	
	public:
	
	OAMSprite( 
		u8 OAMSpriteIndex, 
		u8* graphicsData, 
		u16 spriteSheetWidthInPixels,
		SpritePool* spritePool,
		Console* console,
		OamState* oam
		);
		
	virtual void SetOAMSpriteIndex( 
		u8 oamSpriteIndex,
		bool forceOffScreen
		);
	
	virtual ~OAMSprite();
	
	void MoveTo( Coords_F position );
	void MoveToUnchecked( Coords_F position );
	
	void ChangeFrame( u8 row, u8 column );
	
	void Render();
	
	Coords_F GetPosition();
	Coords_F GetCentrePosition();
	
	Coords_F GetPositionDelta();
	
	void SetIsFacingRight( bool isFacingRight );
	bool GetIsFacingRight();
	void SetPriorityHigh();
}; 


#endif