#ifndef __PRIORITIZINGSPRITEPOOL__H
#define __PRIORITIZINGSPRITEPOOL__H

#include "OAMSprite.h"

/*
	A sprite pool which accounts for sprite rendering priority. Rendering priority
	is used to determine which sprite is rendered "on top" of others.
	
	This is needed because:
		- more than 4 sprites can be overlapping
		- there are only 4 hardware-defined priorities
		- if sprite priorities are equal for two sprites, that with
			a lower OAM sprite id will have higher priority
*/
class PrioritizingSpritePool : public SpritePool {
	
	private:
	
	// ensures that sprite OAM indexes are set according to their index in our array
	void SetAllOAMIndexes();
	
	public:
	
	PrioritizingSpritePool(
		Console* console,
		u8 capacity
		);
	virtual ~PrioritizingSpritePool();
	
	// determine if reprioritization needs to happen, and performs the reprioritization
	void Reprioritize();
	void FreeSlot( OAMSprite* sprite );
};

#endif