#include <nds.h>
#include "ScrollingBackground.h"
#include "Console.h"

ScrollingBackground::ScrollingBackground(
	Console* console,
	u32* backgroundTileData,
	u32 backgroundTileDataLength,
	u16* backgroundPaletteData,
	u32 backgroundPaletteDataLength,
	u16** backgroundMapChunks,
	u8 chunkCount
	) {

	m_console = console;
	m_mosaicX = 0;
	m_scrollX = 0;	
	m_currentChunk = 0;
	m_chunkCount = chunkCount;
	m_backgroundMapChunks = backgroundMapChunks;
	
	m_bgHandle = bgInit( 0, BgType_Text8bpp, BgSize_T_512x256, 0, 1 );
	
	// lowest priority for our scrolling backgrounds, since we always want sprites visible on top
	bgSetPriority( m_bgHandle, OBJPRIORITY_3 );

	dmaCopy( backgroundTileData, bgGetGfxPtr( m_bgHandle ), backgroundTileDataLength );
	dmaCopy( backgroundPaletteData, BG_PALETTE, backgroundPaletteDataLength );

	m_mapPointer = (u16*) bgGetMapPtr( m_bgHandle );

	for( int y = 0; y < 24; y++ ) {
		dmaCopy( &m_backgroundMapChunks[0][ y * 64 ], &m_mapPointer[ y * 32 ], 32 * 2 );
	}

	m_currentMap = m_backgroundMapChunks[ m_currentChunk ];
	
	bgMosaicEnable( m_bgHandle );
}

ScrollingBackground::~ScrollingBackground() {
}

void ScrollingBackground::ScrollBy( u8 amount ) {
	
	m_scrollX += amount;
	
	if( m_scrollX >= m_chunkCount * 512 - 256 ) {
		// if we're past the end, bound the scroll amount
		m_scrollX = m_chunkCount * 512 - 256;
	} else {
		// otherwise, see if we have to move into the next chunk
		if( ( m_scrollX + 256 ) / 512 > m_currentChunk ) {
			m_currentChunk++;
			m_currentMap = m_backgroundMapChunks[ m_currentChunk ];
			m_console->Debug( "ScrollBy: changing chunk: x=%d chunk=%d", m_scrollX, m_currentChunk );
		}
	}
	
	u32 mapOffset = m_scrollX / 8 + 32;
	u32 layerOffset = ( mapOffset & 63 );
	
	if(layerOffset >= 32) {
		layerOffset += 32 * 32 - 32;
	}
	
	for(int y = 0; y < 24; y++) {
		m_mapPointer[ layerOffset + ( y * 32 ) ] = 
			m_currentMap[ ( mapOffset & 63 ) + ( y * 64 ) ];
	}
	
}

u16 ScrollingBackground::GetScrollAmount() {
	return m_scrollX;
}

void ScrollingBackground::Render() {
	
	bgSetScroll( m_bgHandle, m_scrollX, 0 );
	bgUpdate();
}

bool ScrollingBackground::IsAtEnd() {
	return m_scrollX >= m_chunkCount * 512 - 256;
}

void ScrollingBackground::IncreaseMosaicEffect() {
	m_mosaicX++;
	bgSetMosaic( m_mosaicX, 0 );
}

void ScrollingBackground::ResetMosaicEffect() {
	m_mosaicX = 0;
	bgSetMosaic( m_mosaicX, 0 );
}