#ifndef __SCROLLINGBACKGROUND_H
#define __SCROLLINGBACKGROUND_H

#include <nds.h>
#include "Console.h"

/*
	This class provides access to scrolling backgrounds.
	It uses 512x256 chunks to create a strip background,
	which can only be scrolled to the right, without wrapping around
	once its rightmost pixels are on screen.
	
	The 512x256 chunks can be layers output by Pern Edit, etc.
*/
class ScrollingBackground {

	private:
	
	u16 m_scrollX;
	u8 m_currentChunk;
	
	Console* m_console;
	u8 m_chunkCount;
	u16** m_backgroundMapChunks;
	
	u16* m_currentMap;
	u16* m_mapPointer;
	u32 m_bgHandle;
	
	u16 m_mosaicX;
	
	public:
	
	ScrollingBackground(
		Console* console,
		u32* backgroundTileData,
		u32 backgroundTileDataLength,
		u16* backgroundPaletteData,
		u32 backgroundPaletteDataLength,
		u16** backgroundMapChunks, // pointers to map chunks of 512x256
		u8 chunkCount
		);

	virtual ~ScrollingBackground();
	
	bool IsAtEnd();
	void ScrollBy( u8 amount );
	u16 GetScrollAmount();
	
	void IncreaseMosaicEffect();
	void ResetMosaicEffect();
	
	void Render();
};


#endif