#include <nds.h>
#include <maxmod9.h>
#include "Song.h"
#include "Console.h"

Song::Song( 
	u32 songId,
	Console* console,
	CoreUtils* coreUtils,
	AudioPlayer* audioPlayer
	) {
	
	m_audioPlayer = audioPlayer;
	m_coreUtils = coreUtils;
	m_console = console;
	m_songId = songId;
	m_isPlaying = false;
	
	mmLoad( m_songId );
	m_console->Debug( "Song: %d at %p", m_songId, this );
}
	
Song::~Song() {

	this->Stop();

	mmUnload( m_songId );
	m_audioPlayer->FreeSong( this );
	m_console->Debug( "~Song: %d at %p", m_songId, this );
}
	
void Song::Play() {
	if( m_isPlaying ) {
		return;
	}
	m_isPlaying = true;
	mmSetModuleVolume( 600 ); // volume must be in range 0 -> 1024
	mmStart( m_songId, MM_PLAY_LOOP );
	m_console->Debug( "Play: %d at %p", m_songId, this );
}

void Song::Stop() {
	if( !m_isPlaying ) {
		return;
	}
	
	m_isPlaying = false;
	
	m_coreUtils->Sleep( 1 );
	mmStop();
	m_console->Debug( "Stop: %d at %p", m_songId, this );
}
