#include <maxmod9.h>
#include "SoundEffect.h"
#include "Console.h"

SoundEffect::SoundEffect( 
	u32 soundEffectId,
	u8 multiplexingLimit,
	Console* console
	) {

	m_console = console;
	m_soundEffectId = soundEffectId;
	m_multiplexingLimit = multiplexingLimit;
	m_nextMultiplexingSlot = 0;
	
	mmLoadEffect( soundEffectId );
	
	// sound effect handles (used for stopping)
	m_handles = new mm_sfxhand* [ m_multiplexingLimit ];
	// descriptors
	m_descriptors = new mm_sound_effect* [ m_multiplexingLimit ];
	for( u8 i = 0; i < m_multiplexingLimit; i++ ) {
		m_descriptors[ i ] = new mm_sound_effect;
		
		m_descriptors[ i ]->id = soundEffectId; // sample ID (make sure it is loaded);
		m_descriptors[ i ]->rate = 1024; // playback rate, 1024 = original sound
		m_descriptors[ i ]->handle = 0; // 0 = allocate new handle
		m_descriptors[ i ]->volume = 255; // volume level ( 0 - 255 )
		m_descriptors[ i ]->panning = 128; // panning = center
		
		m_handles[ i ] = NULL;
	}
	
	m_console->Debug( "SoundEffect: at %p", this );
}

SoundEffect::~SoundEffect() {

	for( u8 i = 0; i < m_multiplexingLimit; i++ ) {
		mmEffectCancel( *m_handles[ i ] ); // stop
		mmEffectRelease( *m_handles[ i ] ); // release handle
		delete m_descriptors[ i ];
		
		delete m_handles[ i ];
	}
	
	delete m_descriptors;
	delete m_handles;

	mmUnloadEffect( m_soundEffectId );
	
	m_console->Debug( "~SoundEffect: at %p", this );
}

void SoundEffect::Play() {
	
	// generate and save a new handle if one is not present in that slot
	if( NULL == m_handles[ m_nextMultiplexingSlot ] ) {		
		m_handles[ m_nextMultiplexingSlot ] = new mm_sfxhand;
	}
	*m_handles[ m_nextMultiplexingSlot ] = mmEffectEx( 
		m_descriptors[ m_nextMultiplexingSlot ] 
	);

	
	m_console->Debug( "[SoundEffect]Play: at %p, slot %d", this, m_nextMultiplexingSlot );
	
	// move to the next slot
	m_nextMultiplexingSlot++;
	
	if( m_nextMultiplexingSlot >= m_multiplexingLimit ) {
		m_nextMultiplexingSlot = 0;
	}
}

void SoundEffect::Stop() {
	
	for( u8 i = 0; i < m_multiplexingLimit; i++ ) {
		// if we have played back from this slot, then stop it
		if( NULL != m_handles[ i ] ) {
			mmEffectCancel( *m_handles[ i ] );
		}
	}
	
	m_console->Debug( "[SoundEffect]Stop: at %p", this );
}
