#ifndef __SOUNDEFFECT_H
#define __SOUNDEFFECT_H

#include <nds.h>
#include <maxmod9.h>
#include "Console.h"

/*
	Provides multiplexing sound effects. When constructed with a multiplexing limit
	greater than one, the sound can be played multiple times simultaneously, up to
	as many times as specified by the limit, or limited by the hardware.
	The intention of this construct is to be used for sounds which are played back
	more than one at a time, such as punches landing from different opponents while
	fighting, etc.
*/
class SoundEffect {

	private:
	
	Console* m_console;
	
	u32 m_soundEffectId; // as created via make, this is the payload which is played by this effect
	u8 m_multiplexingLimit;	// maximum number of synchronous playbacks for this effect
	
	u8 m_nextMultiplexingSlot; // index of next slot we will use to play next
	
	mm_sound_effect** m_descriptors; // used to describe playback options (volume, panning, etc.)
	mm_sfxhand** m_handles; // handle of a [payload, descriptor] pair
	
	public:
	
	SoundEffect( 
		u32 soundEffectId,
		u8 multiplexingLimit,
		Console* console
		);
	
	virtual ~SoundEffect();
	
	void Play();
	
	void Stop();
};

#endif