#include "OAMSprite.h"
#include "SubScreenOAMSprite.h"
#include "MainScreenOAMSprite.h"
#include "SpritePool.h"


SpritePool::SpritePool( 
	Console* console,
	u8 capacity
	) {

	m_capacity = capacity;
	m_console = console;
	
	m_sprites = new OAMSprite* [ m_capacity ];

	// a NULL value in the sprites array represents an available sprite spot
	for( u8 i = 0; i < m_capacity; i++ ) {
		m_sprites[ i ] = NULL;
	}
	
	m_console->Debug( "SpritePool: capacity:%d at %p", m_capacity, this );
}
	
SpritePool::~SpritePool() {
	// destroy any present sprites
	for( u8 i = 0; i < m_capacity; i++ ) {
		if( NULL != m_sprites[ i ] ) {
			delete m_sprites[ i ];
			m_sprites[ i ] = NULL;
		}
	}
	
	delete m_sprites;
	m_console->Debug( "~SpritePool: %p", this );
}

u8 SpritePool::FindAvailableSlot() {
	
	for( u8 i = 0 ; i < m_capacity; i++ ) {
		if( NULL == m_sprites[ i ] ) {
			m_console->Debug( "FindAvailableSlot: found [%d]", i );
			return i;
		}
	}
	
	m_console->Error( "FindAvailableSlot: Could not find an empty slot!" );
	
	if( NULL != m_sprites[ m_capacity - 1 ] ) {
		m_console->Error( "FindAvailableSlot: Destroyed %p!", m_sprites[ m_capacity - 1 ] );
		delete m_sprites[ m_capacity - 1 ];
		m_sprites[ m_capacity - 1 ] = NULL;
	}
	return m_capacity - 1;
}

void SpritePool::FreeSlot( OAMSprite* sprite ) {
	
	u8 freedCount = 0;
	for( u8 i = 0; i < m_capacity; i++ ) {
	
		// compare pointers
		if( sprite == m_sprites[ i ] ) {
			freedCount++;
			m_sprites[ i ] = NULL;
			m_console->Debug( "FreeSlot: %p [%d]", sprite, i );
		}
	}
	
	if( freedCount == 0 ) {
		m_console->Error( "FreeSlot: Not found!" );
	}
}
	
void SpritePool::AddSprite( OAMSprite* sprite ) {
	
	u8 spriteIndex = FindAvailableSlot();
	if( NULL != m_sprites[ spriteIndex ] ) {
		delete m_sprites[ spriteIndex ];
	}
	
	m_sprites[ spriteIndex ] = sprite;
		
	m_console->Debug( 
		"AddSprite: %p [%d]", 
		m_sprites[ spriteIndex ],
		spriteIndex
		);
}