#include "ChargingEnemyDriver.h"

ChargingEnemyDriver::ChargingEnemyDriver(
	Console* console,
	CoreUtils* coreUtils,
	Character* character,
	Character* enemy,
	Impact* impact,
	bool alwaysKnockDown
	) : SimpleEnemyDriver( 
		console, 
		coreUtils,
		character,
		enemy,
		impact,
		alwaysKnockDown
		) {
		
	m_minimumChargeDistance = 80.0f;
	m_chanceOutOfThousandToCharge = character->GetProperties()->GetChanceToCharge();
}

ChargingEnemyDriver::~ChargingEnemyDriver() {
}

void ChargingEnemyDriver::Decision() {
	
	m_enemy->SetLastAggressor( NULL );
	UtilitiesCompositeActor* myActor = m_character->GetActor();
	UtilitiesCompositeActor* enemyActor = m_enemy->GetActor();
	
	if( myActor->IsCharging() ) {
		if( myActor->IsFacingWithinDistanceOf( enemyActor, m_hittingRadius ) &&
			enemyActor->CanFallDownOrRecoverFromHit()
			) {

			m_enemy->GetSoundboard()->WasThrownDown();
			
			m_enemy->GetProperties()->DecreaseLifeBy( 
				m_character->GetProperties()->GetChargingDamage()
				);
			m_enemy->SetLastAggressor( m_character );
			bool hasDied = m_enemy->GetProperties()->GetLife() <= 0;
			
			enemyActor->SetDisplacementTendency( myActor->IsToTheLeftOf( enemyActor ) );
			if( hasDied ) {
				enemyActor->SetHasDied();
			}
			enemyActor->FallDown();
				
			m_impact->ShowForPosition( myActor->GetPosition(), myActor->IsFacingRight() );
		}
		
		return;
	}
	
	bool canEnemyBeHit = enemyActor->CanFallDownOrRecoverFromHit();
	if( !canEnemyBeHit ) {
		myActor->Stand();
		return;
	}
	
	float distance = myActor->DistanceTo( enemyActor );
	if( distance >= m_minimumChargeDistance ) {
		
		bool madeChargeRoll = m_coreUtils->Random( 1000 ) < m_chanceOutOfThousandToCharge;
		
		if( myActor->CanCharge() && madeChargeRoll ) {
			m_character->GetSoundboard()->StartCharging();
			myActor->Charge( 
				enemyActor->GetPosition(),
				m_character->GetProperties()->GetChargingSpeed()
				);
			return;
		}
	}
	
	SimpleEnemyDriver::Decision();
}
