#include "DriverFactory.h"

DriverFactory::DriverFactory(
	Console* console,
	CoreUtils* coreUtils,
	InputManager* inputManager,
	Impact* impact
	) {
	
	m_console = console;
	m_coreUtils = coreUtils;
	m_inputManager = inputManager;
	m_impact = impact;
	
	m_lastPlayerCharacter = NULL;
}

DriverFactory::~DriverFactory() {
}
	
Driver* DriverFactory::Create(
	Character* character,
	DriverTypes type,
	List<Character>* nonPlayerCharacters,
	Scroller* scroller
	) {
	
	if( PLAYER_DRIVER != type && NULL == m_lastPlayerCharacter ) {
		m_console->Error( "[DriverFactory:Create] Must create a player driver before any non-player drivers" );
	}
	
	bool alwaysKnockDown = 
		type == AGGRESSIVE_SIMPLE_ENEMY_DRIVER ||
		type == AGGRESSIVE_CHARGING_ENEMY_DRIVER ||
		type == AGGRESSIVE_FLANKING_ENEMY_DRIVER;
	
	switch( type ) {
		case PLAYER_DRIVER:
			m_lastPlayerCharacter = character;
			return new PlayerDriver(
				m_coreUtils,
				m_console,
				character,
				m_inputManager,
				nonPlayerCharacters,
				scroller,
				m_impact
				);
		case SIMPLE_ENEMY_DRIVER:
		case AGGRESSIVE_SIMPLE_ENEMY_DRIVER:
			return new SimpleEnemyDriver( 
				m_console,
				m_coreUtils,
				character,
				m_lastPlayerCharacter,
				m_impact,
				alwaysKnockDown
				);
		case CHARGING_ENEMY_DRIVER:
		case AGGRESSIVE_CHARGING_ENEMY_DRIVER:
			return new ChargingEnemyDriver(
				m_console,
				m_coreUtils,
				character,
				m_lastPlayerCharacter,
				m_impact,
				alwaysKnockDown
				);
		case FLANKING_ENEMY_DRIVER:
		case AGGRESSIVE_FLANKING_ENEMY_DRIVER:
			return new FlankingEnemyDriver(
				m_console,
				m_coreUtils,
				character,
				m_lastPlayerCharacter,
				m_impact,
				alwaysKnockDown
				);
	}
	
	m_console->Error( "[DriverFactory::Create]: Unrecognized driver type" );
	return NULL;
}
