#include "FlankingEnemyDriver.h"

FlankingEnemyDriver::FlankingEnemyDriver(
	Console* console,
	CoreUtils* coreUtils,
	Character* character,
	Character* enemy,
	Impact* impact,
	bool alwaysKnockDown
	) : SimpleEnemyDriver( 
		console, 
		coreUtils,
		character,
		enemy,
		impact,
		alwaysKnockDown
		) {
		
	m_incrementCount = 0;
}

FlankingEnemyDriver::~FlankingEnemyDriver() {
}

bool FlankingEnemyDriver::TryFlank() {

	UtilitiesCompositeActor* myActor = m_character->GetActor();
	UtilitiesCompositeActor* enemyActor = m_enemy->GetActor();
	
	Coords_F myCentrePosition = myActor->GetCentrePosition();
	Coords_F enemyCentrePosition = enemyActor->GetCentrePosition();

	// am I too close vertically to flank?
	float verticalDistance = fabs( myCentrePosition.y - enemyCentrePosition.y );
	if( verticalDistance < MINIMUM_VERTICAL_DISTANCE_TO_BEGIN_FLANKING ) {
		return false;
	}
	
	// am I too close horizontally to flank?
	float horizontalDistance = fabs( myCentrePosition.x - enemyCentrePosition.x );
	if( horizontalDistance <= HORIZONTAL_OFFSET_BEHIND_TARGET ) {
		return false;
	}
	
	// is the enemy not facing me already?
	bool isEnemyFacingMe = enemyActor->IsFacing( myActor );
	if( !isEnemyFacingMe ) {
		return false;
	}
		
	// target an area near the enemy
	float targetXDelta = HORIZONTAL_OFFSET_BEHIND_TARGET;
	if( enemyActor->IsFacingRight() ) {
		targetXDelta = -1.0f * targetXDelta;
	}
	
	// make a roll
	bool madeRoll = m_coreUtils->Random( 1000 ) < CHANCE_TO_FLANK;
	if( !madeRoll ) {
		return false;
	}
	
	m_console->Debug( "Flanking!" );
	
	// target an area behind the enemy
	Coords_F targetPosition( 
		enemyActor->GetPosition().x + targetXDelta,
		myActor->GetPosition().y
		);
	
	// walk a bit faster when flanking
	float increasedSpeed = m_walkingSpeedUnitsPerFrame * 1.5f;
	m_increment = myActor->DeltaTo( targetPosition, increasedSpeed );
	float totalLength = ( targetPosition - myActor->GetPosition() ).GetLength();
	m_incrementCount = (u16)( totalLength / m_increment.GetLength() );
	
	return true;
}

void FlankingEnemyDriver::Decision() {
	
	UtilitiesCompositeActor* myActor = m_character->GetActor();
	UtilitiesCompositeActor* enemyActor = m_enemy->GetActor();
	
	bool canEnemyBeHit = enemyActor->CanFallDownOrRecoverFromHit();
	if( !canEnemyBeHit ) {
		myActor->Stand();
		return;
	}
	
	// am I currently flanking?
	if( myActor->CanMoveOrStand() && m_incrementCount > 0 ) {
		m_incrementCount--;
		myActor->MoveBy( m_increment );
		return;
	}
	
	// try to begin flanking
	if( myActor->CanMoveOrStand() && this->TryFlank() ) {
		return;
	}
	
	// default behaviour
	SimpleEnemyDriver::Decision();
}
