#ifndef __PLAYER_DRIVER_H
#define __PLAYER_DRIVER_H

#include "../core/core.h"
#include "../actors/actors.h"
#include "../backgrounds/backgrounds.h"
#include "../overlays/overlays.h"
#include "Driver.h"

/*
	Rely on input (buttons, d-pad) to drive an actor
*/
class PlayerDriver : public Driver {
	
	private:
	
	InputManager* m_inputManager;
	float m_walkingSpeedUnitsPerFrame;
	List<Character>* m_enemies;
	Scroller* m_scroller;
	
	static const float SCROLLING_THRESHOLD = 100.0f;
	
	// if at least this many frames pass since a hit occurrence event is caught
	// and the next player input, don't continue the combo
	static const u8 FRAME_COUNT_ALLOWED_WITH_NO_COMBO_CONTINUATION_PRESS = 30;
	static const s16 ENERGY_GAIN_PER_COMBO_HIT = 6;
	
	u32 m_soundsRemainingForCurrentFrame;
	
	u32 m_lastComboButtonPressFrame;
	u32 m_lastComboHitOccurrenceFrame;
	u32 m_frameCounter;
	Event m_comboContinuationEvent;
	u8 m_comboHitCount;
	float m_hittingRadius;
	Event m_singleHitOccurredEvent;
	Event m_comboHitOccurredEvent;
	bool m_lastHitOccurrenceHitSomeone;
	
	s16 AdjustDamage( s16 damage );
	s16 AdjustEnergyGain( s16 energy );
	
	bool TryComboHitNearbyEnemies();
	void ApplySingleHitOnNearbyEnemies();
	void TurnCharacterIfRequested();
	bool TryAllocateSound();
	
	void HandleSingleHit();
	void HandleComboHit();
	void HandleHeal();
	void HandleMystical();
	void HandleMovement();
	
	public:
	
	PlayerDriver(
		CoreUtils* coreUtils,
		Console* console,
		Character* character,
		InputManager* inputManager,
		List<Character>* enemies,
		Scroller* scroller,
		Impact* impact
		);

	virtual ~PlayerDriver();
	
	void Decision();
};

#endif