#include "EnemyView.h"

EnemyView::EnemyView( 
	Console* console,
	GaugeFactory* gaugeFactory,
	MugShotFactory* mugShotFactory,
	Fighter* fighter
	) {
	
	m_fighter = fighter;
	m_console = console;
	
	m_lifeGauge = gaugeFactory->Create( 2.0f );
	Coords_F mugShotPosition( 88.0f, 5.0f );
	m_mugShot = mugShotFactory->Create( mugShotPosition );
	m_frameCountdown = 0;
	this->Hide();
}

EnemyView::~EnemyView() {
	this->Hide();
	delete m_lifeGauge;
	delete m_mugShot;
}

void EnemyView::Update() {
	
	Character* lastVictim = m_fighter->GetCharacter()->GetLastVictim();
	Character* lastAggressor = m_fighter->GetCharacter()->GetLastAggressor();
	Character* relevant = NULL;
	
	// show victim if there is one; otherwise show aggressor if there is one
	if( NULL != lastVictim ) {
		relevant = lastVictim;
	} else if( NULL != lastAggressor ) {
		relevant = lastAggressor;
	} else {
		// if there isn't a remnant on screen, hide the whole thing
		if( m_frameCountdown == 0 ) {
			this->Hide();
		}
		return;
	}
	
	m_frameCountdown = VISIBLE_FRAME_COUNT;
	m_lifeGauge->SetValue(
		relevant->GetProperties()->GetLife(),
		relevant->GetProperties()->GetMaxLife()
		);
	m_mugShot->SetType( relevant->GetType() );
	
	m_console->PrintAt( 
		CHARACTER_NAME_COLUMN, 
		CHARACTER_NAME_ROW, 
		"%s", relevant->GetName() 
		);
	m_console->PadBlankLine();
}

void EnemyView::Hide() {
	m_lifeGauge->Hide();
	m_mugShot->Hide();
	m_console->MoveCursorTo( 
		0, 
		CHARACTER_NAME_ROW
		);
	m_console->PadBlankLine();
}

void EnemyView::Render() {
	if( m_frameCountdown == 0 ) {
		return;
	}
	
	m_frameCountdown--;
	m_lifeGauge->Render();
	m_mugShot->Render();
	
	if( m_frameCountdown == 0 ) {
		this->Hide();
	}
}