#include "EntryPoint.h"

EntryPoint::EntryPoint() {
	m_context = new SystemContext();
	m_context->Initialize();
	
	m_coreUtils = new CoreUtils();
	m_console = new Console( m_coreUtils, false );
	
	char author[] = "Geoincursion was written by Sebastian Mihai, 2014, http://sebastianmihai.com";
	m_console->Print( author );
	m_console->Clear();
	
	m_audioPlayer = new AudioPlayer( m_console, m_coreUtils );
	
	m_soundBoardFactory = new SoundboardFactory( m_coreUtils, m_console, m_audioPlayer );
	
	m_levelFactory = new LevelFactory(
		m_console,
		m_coreUtils,
		m_soundBoardFactory
		);
		
	m_controllerFactory = new ControllerFactory(
		m_console,
		m_audioPlayer,
		m_levelFactory
		);
	
	m_controllerTypes[ 0 ] = MENU_CONTROLLER;
	m_controllerTypes[ 1 ] = ROMANIA_CONTROLLER;
	m_controllerTypes[ 2 ] = USA_CONTROLLER;
	m_controllerTypes[ 3 ] = CHINA_CONTROLLER;
	m_controllerTypes[ 4 ] = BRITAIN_CONTROLLER;
	m_controllerTypes[ 5 ] = CORE_CONTROLLER;
}

EntryPoint::~EntryPoint() {
	delete m_controllerFactory;
	delete m_levelFactory;
	delete m_soundBoardFactory;
	
	delete m_audioPlayer;
	
	delete m_console;
	delete m_coreUtils;
	delete m_context;
}
	
void EntryPoint::Run() {
	FrameRunnerResult result( 0, 0 );
	
	while( 1 ) {
	
		for( u8 i = 0; i < CONTROLLER_COUNT; i++ ) {
			Controller* controller = m_controllerFactory->Create( 
				m_controllerTypes[ i ],
				result
				);
			result = controller->Run();
			delete controller;
		}
	}
}

