#include "FighterFactory.h"

FighterFactory::FighterFactory(
	Console* console,
	CompositeActorFactory* compositeActorFactory,
	CharacterFactory* characterFactory,
	DriverFactory* driverFactory
	) {
	
	m_console = console;
	m_compositeActorFactory = compositeActorFactory;
	m_characterFactory = characterFactory;
	m_driverFactory = driverFactory;
}

FighterFactory::~FighterFactory() {
}
	
Fighter* FighterFactory::Helper_Create(
	FighterTypes type,
	Scroller* scroller,
	Coords_F position
	) {
	
	UtilitiesCompositeActor* actor;
	Character* character;
	Driver* driver;
	
	switch( type ) {
				
		case SMECHER_FIGHTER_WEAK_FLANKING:
			actor = m_compositeActorFactory->Create( ACTOR_SMECHER );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, SMECHER_CHARACTER );
			driver = m_driverFactory->Create( 
				character, 
				FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case SMECHER_FIGHTER_WEAK:
			actor = m_compositeActorFactory->Create( ACTOR_SMECHER_GREEN );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, SMECHER_CHARACTER_GREEN );
			driver = m_driverFactory->Create( 
				character, 
				SIMPLE_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case SMECHER_FIGHTER_STRONG:
			actor = m_compositeActorFactory->Create( ACTOR_SMECHER_BLUE );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, SMECHER_CHARACTER_BLUE );
			driver = m_driverFactory->Create( 
				character, 
				SIMPLE_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case SMECHER_FIGHTER_STRONG_FLANKING:
			actor = m_compositeActorFactory->Create( ACTOR_SMECHER_PURPLE );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, SMECHER_CHARACTER_PURPLE );
			driver = m_driverFactory->Create( 
				character, 
				FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case SMECHER_FIGHTER_BOSS:
			actor = m_compositeActorFactory->Create( ACTOR_SMECHER_RED_NOSTUNLOCK );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, SMECHER_CHARACTER_RED );
			driver = m_driverFactory->Create( 
				character, 
				FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
			
		case CUTITAR_FIGHTER:
			actor = m_compositeActorFactory->Create( ACTOR_CUTITAR );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, CUTITAR_CHARACTER );
			driver = m_driverFactory->Create( 
				character, 
				FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case CUTITAR_FIGHTER_BOSS:
			actor = m_compositeActorFactory->Create( ACTOR_CUTITAR_NOSTUNLOCK );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, CUTITAR_CHARACTER_BOSS );
			driver = m_driverFactory->Create( 
				character, 
				CHARGING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
				
		case MASCAT_FIGHTER_WEAK_FLANKING:
			actor = m_compositeActorFactory->Create( ACTOR_MASCAT );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MASCAT_CHARACTER );
			driver = m_driverFactory->Create( 
				character, 
				FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case MASCAT_FIGHTER_STRONG_CHARGING:
			actor = m_compositeActorFactory->Create( ACTOR_MASCAT_GREEN );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MASCAT_CHARACTER_GREEN );
			driver = m_driverFactory->Create( 
				character, 
				CHARGING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case MASCAT_FIGHTER_BOSS:
			actor = m_compositeActorFactory->Create( ACTOR_MASCAT_RED_NOSTUNLOCK );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MASCAT_CHARACTER_RED );
			driver = m_driverFactory->Create( 
				character, 
				AGGRESSIVE_FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);
			return new Fighter( 
				m_console,
				character,
				driver
				);
				
		case MEISS_FIGHTER_NONCHARGING:
			actor = m_compositeActorFactory->Create( ACTOR_MEISS_DARK );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MEISS_CHARACTER_DARK );
			driver = m_driverFactory->Create( 
				character, 
				AGGRESSIVE_FLANKING_ENEMY_DRIVER,
				NULL,
				scroller
				);		
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case MEISS_FIGHTER_CHARGING:
			actor = m_compositeActorFactory->Create( ACTOR_MEISS );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MEISS_CHARACTER );
			driver = m_driverFactory->Create( 
				character, 
				AGGRESSIVE_CHARGING_ENEMY_DRIVER,
				NULL,
				scroller
				);		
			return new Fighter( 
				m_console,
				character,
				driver
				);
		case MEISS_FIGHTER_BOSS:
			actor = m_compositeActorFactory->Create( ACTOR_MEISS_NOSTUNLOCK );
			actor->RelocateTo( position );
			character = m_characterFactory->Create( actor, MEISS_CHARACTER_BOSS );
			driver = m_driverFactory->Create( 
				character, 
				AGGRESSIVE_CHARGING_ENEMY_DRIVER,
				NULL,
				scroller
				);		
			return new Fighter( 
				m_console,
				character,
				driver
				);
	}
	
	m_console->Error( "[FighterFactory::Create]: Unrecognized fighter type" );
	return NULL;
}


Fighter* FighterFactory::CreatePlayer(
	Scroller* scroller,
	s16 initialLife,
	s16 initialEnergy
	) {
	
	Coords_F position( 20.0f, 100.0f );

	UtilitiesCompositeActor* actor = m_compositeActorFactory->Create( ACTOR_PLAYER );
	actor->RelocateTo( position );
	Character* character = m_characterFactory->Create( 
		actor, 
		PLAYER_CHARACTER,
		initialLife,
		initialEnergy
		);
	Driver* driver = m_driverFactory->Create( 
		character, 
		PLAYER_DRIVER,
		m_characterFactory->GetNonPlayerCharacterPool(),
		scroller
		);

	return new Fighter( 
		m_console,
		character,
		driver
		);
}

Fighter* FighterFactory::CreateFromSpawn(
	Spawn* spawn,
	Scroller* scroller
	) {
	
	Fighter* result = this->Helper_Create( 
		spawn->Type, 
		scroller,
		spawn->InitialPosition
		);
	
	return result;
}
