#ifndef __FRAMEDESCRIPTOR_H
#define __FRAMEDESCRIPTOR_H

#include <nds.h>

/*
	Used to describe available frames from a sprite sheet
*/
class FrameDescriptor {

	public:
	
	virtual ~FrameDescriptor();
	
	// a phase is a "horizontal" strip of frames, representing
	// an action, such as walking, or hitting.
	// while sprite sheets are rectangular, they could be jagged, in that, for example
	// the first phase (row) has three animation frames, while
	// the second phase has four animation frames
	//
	// naturally, classes derived from this abstract base class are tightly coupled to the
	// graphics characteristics themselves
	
	// returns the number of phases
	virtual u8 GetPhaseCount() = 0;
	
	// returns length in frames for specified phase
	virtual u8 GetPhaseFramesCount( u8 phase ) = 0;
	
	// returns count of video frames per animation frame
	// for specified phase
	virtual u8 GetPhaseVideoFramePerAnimationFrame( u8 phase ) = 0;
};

#endif