#include "ReversedGauge.h"
#include "spr_subscreen.h"

ReversedGauge::ReversedGauge( 
	Coords_F position,
	SpriteManager* spriteManager
	) : Gauge( position ) {
	
	m_spriteManager = spriteManager;
	
	m_primary = spriteManager->AllocateSubScreenSprite();
	m_primary->ChangeFrame( 1, 0 );
	m_secondary = spriteManager->AllocateSubScreenSprite();
	m_secondary->ChangeFrame( 1, 0 );

	Coords_F primaryPosition;
	primaryPosition.x = m_position.x + 64.0f;
	primaryPosition.y = m_position.y;
	m_primaryFrame = spriteManager->AllocateSubScreenSprite();
	m_primaryFrame->MoveToUnchecked( primaryPosition );
	m_primaryFrame->ChangeFrame( 0, 1 );
	
	m_secondaryFrame = spriteManager->AllocateSubScreenSprite();
	m_secondaryFrame->ChangeFrame( 0, 2 );
	m_secondaryFrame->MoveToUnchecked( m_position );
	m_secondaryFrame->SetIsFacingRight( false );
	
	this->SetValue( 0, 100 );
}

ReversedGauge::~ReversedGauge() {
	delete m_primary;
	delete m_secondary;
	delete m_primaryFrame;
	delete m_secondaryFrame;
}

void ReversedGauge::SetValue( 
	s16 current,
	s16 outOf
	) {
	
	Gauge::SetValue( current, outOf );
	
	if( m_value <= 64 ) {
		// shift primary left as much as needed
		m_primaryPosition = m_position;
		m_primaryPosition.x = m_primaryPosition.x + 64.0f + 64.0f - (float)m_value;
		
		// move secondary off-screen
		m_secondaryPosition.x = 256.0f;
		m_secondaryPosition.y = 0.0f;
	} else {
		// shift primary to initial position
		m_primaryPosition = m_position;
		m_primaryPosition.x = m_primaryPosition.x + 64.0f;
		
		// shift secondary left as much as needed
		m_secondaryPosition = m_position;
		m_secondaryPosition.x = m_secondaryPosition.x + 64.0f + 64.0f - (float)m_value;
	}
	
}

void ReversedGauge::Render() {
	m_primary->MoveToUnchecked( m_primaryPosition );
	m_primary->Render();
	m_secondary->MoveToUnchecked( m_secondaryPosition );
	m_secondary->Render();
	
	m_primaryFrame->Render();
	m_secondaryFrame->Render();
}

void ReversedGauge::Hide() {
	m_primary->ChangeFrame( 0, 3 );
	m_secondary->ChangeFrame( 0, 3 );
	m_primaryFrame->ChangeFrame( 0, 3 );
	m_secondaryFrame->ChangeFrame( 0, 3 );
}