#ifndef __LEVEL_H
#define __LEVEL_H

#include <nds.h>
#include "../core/core.h"
#include "../animators/animators.h"
#include "../actors/actors.h"
#include "../drivers/drivers.h"
#include "../characters/characters.h"
#include "../fighters/fighters.h"
#include "../backgrounds/backgrounds.h"
#include "../spawning/spawning.h"
#include "FighterListEmptyFuncBool.h"
#include "../gauges/gauges.h"
#include "../movehints/movehints.h"
#include "../playerviews/playerviews.h"
#include "../enemyviews/enemyviews.h"
#include "../overlays/overlays.h"
#include "../soundboards/soundboards.h"

/*
	A controller for one level
*/
class Level : public FrameRunner {
	
	private:

	Console* m_console;
	StageBackground* m_background;
	SpriteManager* m_spriteManager;
	InputManager* m_inputManager;
	CoreUtils* m_coreUtils;
	FighterFactory* m_fighterFactory;
	SpawnRegistry* m_spawnRegistry;
	Impact* m_impact;
	Go* m_go;
	UISoundboard* m_soundboard;
	
	Fighter* m_playerFighter;
	List<Fighter>* m_enemies;
	FighterListEmptyFuncBool* m_fighterListEmptyFuncBool;
	
	PlayerView* m_playerView;
	EnemyView* m_enemyView;
	
	bool m_isPaused;
	u8 m_outroFrames;
	bool m_isDone;
	FrameRunnerResult m_result;
	
	void RemoveDeadEnemies();
	
	protected:
	
	bool IsDone();
	
	public:
	
	Level(
		Console* console,
		BackgroundFactory* backgroundFactory,
		SpriteManager* spriteManager,
		InputManager* inputManager,
		FighterFactory* fighterFactory,
		SpawnRegistryFactory* spawnRegistryFactory,
		PlayerViewFactory* playerViewFactory,
		EnemyViewFactory* enemyViewFactory,
		BackgroundTypes backgroundType,
		SpawnRegistryTypes spawnRegistryType,
		FrameRunnerResult previousRunnerResult,
		Impact* impact,
		Go* go,
		SoundboardFactory* soundBoardFactory
		);
	virtual ~Level();
	
	FrameRunnerResult GetResult();
	void Pre();
	void Frame();
	void Post();
};

#endif