#include <nds.h>

#include "LevelFactory.h"

LevelFactory::LevelFactory(
	Console* console,
	CoreUtils* coreUtils,
	SoundboardFactory* soundBoardFactory
	) {
	m_console = console;
	m_coreUtils = coreUtils;
	
	m_inputManager = new InputManager( console );
	m_soundBoardFactory = soundBoardFactory;
	
	m_spriteManager = new SpriteManager( console );
	m_impact = new Impact( m_spriteManager, coreUtils );
	m_animatorFactory = new AnimatorFactory( console, m_spriteManager );
	
	m_go = new Go( console, m_spriteManager, m_animatorFactory );
	
	m_fallingDownActorFactory = new FallingDownActorFactory( coreUtils, console );
	m_singleActionActorFactory = new SingleActionActorFactory( coreUtils, console );
	m_hitRecoveringActorFactory = new HitRecoveringActorFactory( console, m_singleActionActorFactory );
	m_comboActorFactory = new ComboActorFactory( console, m_singleActionActorFactory );
	m_chargingActorFactory = new ChargingActorFactory( console );
	
	m_compositeActorFactory = new CompositeActorFactory( 
		coreUtils,
		console, 
		m_animatorFactory,
		m_fallingDownActorFactory,
		m_hitRecoveringActorFactory,
		m_singleActionActorFactory,
		m_comboActorFactory,
		m_chargingActorFactory
		);

	m_characterFactory = new CharacterFactory( console, m_soundBoardFactory );

	m_driverFactory = new DriverFactory( 
		console, 
		coreUtils, 
		m_inputManager,
		m_impact
		);
	
	m_fighterFactory = new FighterFactory( 
		console,
		m_compositeActorFactory,
		m_characterFactory,
		m_driverFactory
		);	
		
	m_backgroundFactory = new BackgroundFactory( console );
	
	m_spawnRegistryFactory = new SpawnRegistryFactory( console );
	
	m_mugShotFactory = new MugShotFactory( m_spriteManager );
	m_gaugeFactory = new GaugeFactory( m_spriteManager );
	m_moveHintFactory = new MoveHintFactory( m_spriteManager );
	m_playerViewFactory = new PlayerViewFactory(
		console,
		m_gaugeFactory,
		m_moveHintFactory,
		m_mugShotFactory
		);
	m_enemyViewFactory = new EnemyViewFactory(
		console,
		m_gaugeFactory,
		m_mugShotFactory
		);
}

LevelFactory::~LevelFactory() {

	delete m_go;
	delete m_impact;

	delete m_fighterFactory;
	delete m_driverFactory;
	
	delete m_backgroundFactory;
	delete m_characterFactory;
	
	delete m_compositeActorFactory;
	delete m_chargingActorFactory;
	delete m_comboActorFactory;
	delete m_hitRecoveringActorFactory;
	delete m_singleActionActorFactory;
	delete m_fallingDownActorFactory;
	
	delete m_animatorFactory;
	delete m_spriteManager;
	delete m_inputManager;
	
	delete m_spawnRegistryFactory;
	
	delete m_enemyViewFactory;
	delete m_playerViewFactory;
	delete m_gaugeFactory;
	delete m_moveHintFactory;
	delete m_mugShotFactory;
}

Level* LevelFactory::Create(
	BackgroundTypes backgroundType,
	SpawnRegistryTypes spawnRegistryType,
	FrameRunnerResult previousRunnerResult
	) {
	
	Level* level = new Level(
		m_console,
		m_backgroundFactory,
		m_spriteManager,
		m_inputManager,
		m_fighterFactory,
		m_spawnRegistryFactory,
		m_playerViewFactory,
		m_enemyViewFactory,
		backgroundType,
		spawnRegistryType,
		previousRunnerResult,
		m_impact,
		m_go,
		m_soundBoardFactory
		);
		
	return level;
}

Transition* LevelFactory::CreateTransition(
	BackgroundTypes type
	) {
	Transition* level = new Transition(
		m_coreUtils,
		m_console,
		m_backgroundFactory,
		m_inputManager,
		type
		);
		
	return level;
}
