#ifndef __LEVEL_FACTORY_H
#define __LEVEL_FACTORY_H

#include "../core/core.h"
#include "../animators/animators.h"
#include "../actors/actors.h"
#include "../drivers/drivers.h"
#include "../characters/characters.h"
#include "../fighters/fighters.h"
#include "../backgrounds/backgrounds.h"
#include "../spawning/spawning.h"
#include "../gauges/gauges.h"
#include "../movehints/movehints.h"
#include "../playerviews/playerviews.h"
#include "../mugshots/mugshots.h"
#include "../enemyviews/enemyviews.h"
#include "../overlays/overlays.h"
#include "../soundboards/soundboards.h"
#include "Level.h"
#include "Transition.h"

/*
	Makes levels
*/
class LevelFactory {
	
	private:
	
	CoreUtils* m_coreUtils;
	Console* m_console;
	
	StageBackground* m_background;
	SpriteManager* m_spriteManager;
	InputManager* m_inputManager;
	
	AnimatorFactory* m_animatorFactory;
	FallingDownActorFactory* m_fallingDownActorFactory;
	SingleActionActorFactory* m_singleActionActorFactory;
	HitRecoveringActorFactory* m_hitRecoveringActorFactory;
	ComboActorFactory* m_comboActorFactory;
	ChargingActorFactory* m_chargingActorFactory;
	
	CharacterFactory* m_characterFactory;
	BackgroundFactory* m_backgroundFactory;
	DriverFactory* m_driverFactory;
	FighterFactory* m_fighterFactory;
	
	CompositeActorFactory* m_compositeActorFactory;
	SpawnRegistryFactory* m_spawnRegistryFactory;
	
	MugShotFactory* m_mugShotFactory;
	GaugeFactory* m_gaugeFactory;
	MoveHintFactory* m_moveHintFactory;
	PlayerViewFactory* m_playerViewFactory;
	EnemyViewFactory* m_enemyViewFactory;
	SoundboardFactory* m_soundBoardFactory;
	
	Impact* m_impact;
	Go* m_go;
	
	public:
	
	LevelFactory(
		Console* console,
		CoreUtils* coreUtils,
		SoundboardFactory* soundBoardFactory
		);
	virtual ~LevelFactory();
	
	Level* Create(
		BackgroundTypes backgroundType,
		SpawnRegistryTypes spawnRegistryType,
		FrameRunnerResult previousRunnerResult
		);
	
	Transition* CreateTransition(
		BackgroundTypes type
		);
};

#endif