#include "Transition.h"

Transition::Transition(
	CoreUtils* coreUtils,
	Console* console,
	BackgroundFactory* backgroundFactory,
	InputManager* inputManager,
	BackgroundTypes type
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	m_inputManager = inputManager;
	
	m_trueFuncBool = new TrueFuncBool();
	m_background = backgroundFactory->Create( type, m_trueFuncBool );
	m_isDone = false;
	m_outroFrames = 45;
	
	console->Debug( "Transition: at %p", this );
}

Transition::~Transition() {
	delete m_background;
	delete m_trueFuncBool;
	
	m_console->Debug( "~Transition: at %p", this );
}

void Transition::Pre() {
	if( m_isDone && m_outroFrames == 0 ) {
		return;
	}
	
	if( m_isDone ) {
		m_outroFrames--;
		if( m_outroFrames == 0 ) {
			m_background->ResetMosaicEffect();
		}
		return;
	}

	m_inputManager->Scan();
	if( m_inputManager->IsButtonTapped( BUTTON_START ) ) {
		m_isDone = true;
	}
	
	m_coreUtils->Random();
}

void Transition::Frame() {
	if( m_isDone && m_outroFrames == 0 ) {
		return;
	}
	
	if( m_isDone && m_outroFrames % 5 == 0 ) {
		m_background->IncreaseMosaicEffect();
		return;
	}
	
	m_background->Render();
}

void Transition::Post() {
}

bool Transition::IsDone() {
	return m_isDone && m_outroFrames == 0;
}

FrameRunnerResult Transition::GetResult() {
	FrameRunnerResult result( 100, 100 );
	return result;
}