#include "Impact.h"

Impact::Impact(
	SpriteManager* spriteManager,
	CoreUtils* coreUtils
	) {
	
	m_coreUtils = coreUtils;
	
	m_sprite = spriteManager->AllocateMainScreenSprite( 
		(u8*) impactAnimationTiles, 
		256, 
		(u8*) impactAnimationPal
		);
	m_sprite->ChangeFrame( 0, 0 );
	m_sprite->SetPriorityHigh();
	
	m_frameCounter = 0;
	
	this->Hide();
}

Impact::~Impact() {
	delete m_sprite;
}

void Impact::ShowForPosition( 
	Coords_F position,
	bool isFacingRight
	) {
	
	float shiftXBy = 26.0f;
	if( !isFacingRight ) {
		shiftXBy = shiftXBy * -1.0f;
	}
	
	position.x = position.x + shiftXBy;
	position.y = position.y - 18.0f;
	
	// randomly shift by a few pixels, to create some diversity
	position.x = position.x + (float)m_coreUtils->Random( 9 ) - 4.0f;
	position.y = position.y + (float)m_coreUtils->Random( 9 ) - 4.0f;
	
	m_sprite->MoveToUnchecked( position );
	m_frameCounter = MAX_FRAMES;
}

void Impact::Render() {
	if( m_frameCounter == 0 ) {
		return;
	}
	
	u8 frame = ( MAX_FRAMES - m_frameCounter ) / VIDEO_FRAMES_PER_ANIMATION;
	m_sprite->ChangeFrame( 0, frame );
	m_sprite->Render();
	
	m_frameCounter--;
	if( m_frameCounter == 0 ) {
		this->Hide();
	}
}

void Impact::Hide() {
	Coords_F offscreen( 256.0f, 0.0f );
	m_sprite->MoveToUnchecked( offscreen );
}