#include "PlayerView.h"

PlayerView::PlayerView( 
	Console* console,
	GaugeFactory* gaugeFactory,
	MoveHintFactory* moveHintFactory,
	MugShotFactory* mugShotFactory,
	Fighter* player
	) {
	
	m_console = console;
	m_player = player;
	
	m_lifeGauge = gaugeFactory->Create( 123.0f );
	m_energyGauge = gaugeFactory->CreateReversed( 123.0f );
	
	Coords_F moveHintPosition( 180.0f, 70.0f );
	m_moveHint = moveHintFactory->Create( moveHintPosition, player );
	
	Coords_F mugShotPosition( 96.0f, 120.0f );
	m_mugShot = mugShotFactory->Create( mugShotPosition );
	m_mugShot->SetType( m_player->GetCharacter()->GetType() );
	
	this->Update();
}

PlayerView::~PlayerView() {
	m_lifeGauge->Hide();
	m_energyGauge->Hide();
	m_mugShot->Hide();
	m_moveHint->Hide();
	
	m_console->MoveCursorTo( 
		0, 
		LIFE_ENERGY_ROW
		);
	m_console->PadBlankLine();
	
	delete m_lifeGauge;
	delete m_energyGauge;
	delete m_moveHint;
	delete m_mugShot;
}

void PlayerView::Update() {

	CharacterProperties* properties = m_player->GetCharacter()->GetProperties();

	m_lifeGauge->SetValue(
		properties->GetLife(),
		properties->GetMaxLife()
		);
	
	m_energyGauge->SetValue(
		properties->GetEnergy(),
		properties->GetMaxEnergy()
		);
	
	m_moveHint->SetValue( 
		properties->GetEnergy()
	);
	
	m_console->PrintAt( 
		LIFE_COLUMN, 
		LIFE_ENERGY_ROW, 
		"LIFE"
		);
	m_console->PrintAt( 
		ENERGY_COLUMN, 
		LIFE_ENERGY_ROW, 
		"ENERGY"
		);
}

void PlayerView::Render() {
	m_lifeGauge->Render();
	m_energyGauge->Render();
	m_moveHint->Render();
	m_mugShot->Render();
}