#ifndef __SOUNDBOARD_H
#define __SOUNDBOARD_H

#include <nds.h>
#include "../core/core.h"
#include "SoundHolder.h"

/*
	Builds on the core audio player to allow immediate access to game sound effects,
	without having to worry about their allocation, randomizing from among similar sounds, etc.
*/
class Soundboard {

	private:
	
	CoreUtils* m_coreUtils;

	protected:
	
	SoundHolder* m_soundHolder;
	
	// a utility function which plays one of the sound effects
	// of the provided array, if a check is successful
	void PlayOneWithChance( 
		SoundEffect** effects,
		u16 count,
		u16 chanceOutOfThousand
		);
	// like above, but no check is performed
	void PlayOne( 
		SoundEffect** effects,
		u16 count
		);
		
	// a utility function which plays a sound effect if a 
	// check is successful
	void PlayWithChance( 
		SoundEffect* effect,
		u16 chanceOutOfThousand
		);
	
	public:
	
	Soundboard( 
		CoreUtils* coreUtils,
		SoundHolder* soundHolder
		);
	virtual ~Soundboard();
	
	// the "woosh" sound made by a hit
	virtual void HitWoosh();
	
	// wind up a single hit
	virtual void StartSingleHit();
	// a single hit landed
	virtual void SingleHitLanded();
	
	// wind up a combo hit
	virtual void StartComboHit();
	// a combo hit landed
	virtual void ComboHitLanded();
	
	// the "oof" when someone is thrown down
	virtual void WasThrownDown();
	// some sort of a battle shout just as someone starts charging
	virtual void StartCharging();
	
	// some meaningful accompaniment to a mystical attack
	virtual void Mystical();
	// some meaningful accompaniment to healing
	virtual void Heal();
};

#endif