#include "SoundboardFactory.h"
#include "soundbank.h"

SoundboardFactory::SoundboardFactory( 
	CoreUtils* coreUtils,
	Console* console,
	AudioPlayer* audioPlayer
	) {
	
	m_coreUtils = coreUtils;
	m_console = console;
	
	m_soundHolder = new SoundHolder( 
		m_console, 
		audioPlayer
		);
		
	m_playerSoundboard = new PlayerSoundboard( m_coreUtils, m_soundHolder );
	m_smecherSoundboard = new SmecherSoundboard( m_coreUtils, m_soundHolder );
	m_meissSoundboard = new MeissSoundboard( m_coreUtils, m_soundHolder );
	m_cutitarSoundboard = new CutitarSoundboard( m_coreUtils, m_soundHolder );
	m_mascatSoundboard = new MascatSoundboard( m_coreUtils, m_soundHolder );
	
	m_uiSoundboard = new UISoundboard( m_soundHolder );
}

SoundboardFactory::~SoundboardFactory() {
	
	delete m_smecherSoundboard;
	delete m_meissSoundboard;
	delete m_cutitarSoundboard;
	delete m_mascatSoundboard;
	delete m_playerSoundboard;

	delete m_uiSoundboard;
	
	delete m_soundHolder;
}

Soundboard* SoundboardFactory::Get( SoundboardTypes type ) {
	switch( type ) {
		case PLAYER_SOUNDBOARD:
			return m_playerSoundboard;
		case SMECHER_SOUNDBOARD:
			return m_smecherSoundboard;
		case MEISS_SOUNDBOARD:
			return m_meissSoundboard;
		case CUTITAR_SOUNDBOARD:
			return m_cutitarSoundboard;
		case MASCAT_SOUNDBOARD:
			return m_mascatSoundboard;
	}
	
	m_console->Error( "[SoundboardFactory::Get]: Unrecognized soundboard type" );
	return NULL;
}

UISoundboard* SoundboardFactory::Get() {
	return m_uiSoundboard;
}