#include "SpawnRegistry.h"

SpawnRegistry::SpawnRegistry(
	Console* console,
	u8 spawnCount
	) {
	m_lastCheckedIndex = -1;
	m_console = console;
	m_spawnCount = spawnCount;
	m_spawns = new Spawn* [ m_spawnCount ];
	m_spawns = NULL;
	m_additionCurrentIndex = 0;
}

SpawnRegistry::~SpawnRegistry() {

	if( NULL == m_spawns ) {
		return;
	}

	for( u8 i = 0; i < m_spawnCount; i++ ) {
		delete m_spawns[ i ];
	}
	delete m_spawns;
	
	m_console->Debug( "~SpawnRegistry: at %p", this );
}

void SpawnRegistry::Add( 
	float x,
	float y,
	u16 scrollAmountToSpawn,
	FighterTypes type
	) {

	if( m_additionCurrentIndex >= m_spawnCount ) {
		m_console->Error( "[SpawnRegistry:Add]: adding too many spawns!" );
		return;
	}
	
	m_spawns[ m_additionCurrentIndex ] = new Spawn();
	m_spawns[ m_additionCurrentIndex ]->InitialPosition = Coords_F( x, y );
	m_spawns[ m_additionCurrentIndex ]->ScrollAmountToSpawn = scrollAmountToSpawn;
	m_spawns[ m_additionCurrentIndex ]->Type = type;
	
	m_additionCurrentIndex++;
}

u8 SpawnRegistry::GetAvailableSpawnCount( u16 scrollPosition ) {
	
	if( m_additionCurrentIndex < m_spawnCount - 1 ) {
		m_console->Error( "[GetAvailableSpawnCount]: not yet full!" );
	}
	
	// reached the end
	if( m_lastCheckedIndex >= m_spawnCount - 1 ) {
		return 0;
	}
	
	// count spawns which can still spawn from one over to the right
	// of the current index
	u8 count = 0;
	for( u8 i = m_lastCheckedIndex + 1; i < m_spawnCount; i++ ) {
		if( m_spawns[ i ]->ScrollAmountToSpawn > scrollPosition ) {
			break;
		}
		count++;
	}
	
	return count;
}


Spawn** SpawnRegistry::GetAvailableSpawns( u16 scrollPosition ) {

	if( m_additionCurrentIndex < m_spawnCount - 1 ) {
		m_console->Error( "[GetAvailableSpawnCount]: not yet full!" );
	}

	u8 count = this->GetAvailableSpawnCount( scrollPosition );
	if( 0 == count ) {
		return NULL;
	}
	
	u8 previousLastCheckedIndexPlusOne = m_lastCheckedIndex + 1;
	m_lastCheckedIndex = m_lastCheckedIndex + count;
	
	return &m_spawns[ previousLastCheckedIndexPlusOne ];
}